%%close all;
%%clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; clc; close all; fclose('all'); pause(.001)

% UPLOAD DATA
    CFP_LoadData;

% DEFINE THE SPECIFICATION
    CFP_SPEC;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
% [fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

outfile00 = strcat(spath,'Mh.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior     = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fname = strcat(spath,'dataout',dataset);
% load(fname,'nobsall')
% load(fname,'ti')
% load(fname,'YYall')
% load(fname,'XXall')
% nobs =  nobsall;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates_dns);

%% if JustMode: do not do simulations and compute forecasts at mode
if JustMode
    JustMode
    break;
end

%%  Configure the Metropolis Algorithm
cc0     = 0.2;
cc      = 0.2;


Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
    fixparam
    break
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mh = zeros(1,length(lambdaseq));
percent = .90;

L = 0;
for lambdas = lambdaseq     
    
    L = L+1;
        
	
    %% set params for MH evaluation 
    hmax = 5;
    p = (.1:.1:(9-1)*.1)';
    pcrit = chi2inv(p,npara);
    densfac = -280;
    

    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
	fprintf(1,'\n Time %3.2f ',[ti(I)]);

    fprintf(fid00,'\n Lambda = %2.3f',[lambdas]);

    %% open read files
    %% params file
    infile1 = strcat(spath,'\mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    num1    = nblocks*nsim*npara*8;            % number of bites per file
    numb1   =        nburn*npara*8;            % number of bites to dicard

    %% Posterior file
    infile2 = strcat(spath,'\post',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    num2    = nblocks*nsim*1*8;            % number of bites per file
    numb2   = nburn*1*8;                   % number of bites to dicard

    
    %% get means and st dev of parameters
    fid1   = fopen(infile1,'r');		% input file (set to read)               
    status = fseek(fid1,numb1,'bof');

    theta = [];
	while ( ftell(fid1) < num1 )
        % Read blocks of size nsim
		thetadd = fread(fid1,[nsim,npara],'double');
        if all(size(thetadd,2) == [npara])
            theta = [theta;thetadd];
        else
   			fclose(fid1);
        end
        clear thetadd;
	
	end;
	fclose(fid1);
    ndraws = size(theta,1)
    
    %%%%%%%%%%% Compute posterior moments of the parameters
    thetahat = mean(theta,1)';
    thetasig = cov(theta,1);
    thetasiginv = inv(thetasig);
    thetasiglndet = log(det(thetasig));
   
    thetabands = hpdint(theta,percent,1)';
    
    colnames = {'Parameter ' 'Prior Mean '  'Prior Stdd '  'Post Mean ' strcat(num2str(100*percent),'% Lower Band') ...
            strcat(num2str(100*percent),'% Upper Band') };
	outmat = [pmean,pstdd,thetahat,thetabands];
    
	fprintf(1,'\n');
	fprintf(1,'%4.10s  ',colnames{:});
	for i = 1:length(para_names)
        fprintf(1,'\n %4.10s  ',para_names{i});
        fprintf(1,' %8.4f  ',outmat(i,:));
	end    
    
	fprintf(fid00,'\n');
	fprintf(fid00,'%4.10s  ',colnames{:});
	for i = 1:length(para_names)
        fprintf(fid00,'\n %4.10s  ',para_names{i});
        fprintf(fid00,' %8.4f  ',outmat(i,:));
	end   
    
    %%%%%%%%%%% Compute density estimates
    suminvlike = zeros(length(p),1);
    laginvlike = zeros(hmax,length(p));
    gaminvlike = zeros(hmax,length(p));
    
    
    fid1 = fopen(infile1,'r');		% input file (set to read)               
    status = fseek(fid1,numb1,'bof');
    fid2 = fopen(infile2,'r');		% input file (set to read)               
    status = fseek(fid2,numb2,'bof');

    iblock = 0;
    ndraws = 0;            
    
    while ( ftell(fid1) < num1 ) & ( ftell(fid2) < num2 )             
        iblock = iblock+1;  
        parasimm = fread(fid1,[nsim,npara],'double');
        postsim = fread(fid2,[nsim,1],'double');
        if any(size(parasimm) ~= [nsim,npara]) | any(size(postsim) ~= [nsim,1]) 
   			fclose(fid1);
   			fclose(fid2);
            break;
        else
        

            for j = 1:size(parasimm,1)
                    thetasim = parasimm(j,:)';
                    post = postsim(j);
                    
                    lnftheta = -log(p)-.5*npara*log(2*pi)-.5*thetasiglndet ...
                        -.5*(thetasim-thetahat)'*thetasiginv*(thetasim-thetahat);
                    
                    indtheta = ( (thetasim-thetahat)'*thetasiginv*(thetasim-thetahat) < pcrit );

                    invlike = exp(lnftheta-post+densfac).*indtheta;                    
                    laginvlike = [invlike';laginvlike(1:hmax-1,:)];
                    gaminvlike = gaminvlike + laginvlike.*(ones(hmax,1)*invlike');
                    suminvlike = suminvlike+invlike;
                    
                    ndraws = ndraws+1;
            end
        end
        clear parasimm postsim;
    end
    fclose(fid1);
    fclose(fid2);

    
            
	meaninvlike = suminvlike/ndraws;
	gaminvlike = gaminvlike/ndraws -ones(hmax,1)*(meaninvlike.^2)';
	
	%%% compute standard errors
	suminvlikeerror = gaminvlike(1,:);
	for j = 2 : hmax;
	    suminvlikeerror = suminvlikeerror+2*gaminvlike(j,:)*(1-((j-1)/hmax));
	end
	suminvlikeerror = 100*sqrt(suminvlikeerror/ndraws)./meaninvlike';
	
	pint = densfac-log(meaninvlike);
    pint0 = mean(pint(~isnan(pint) & ~isinf(pint)));
    
    
            
	fprintf(1,'\n marginal data density : %4.7f',pint);
	fprintf(1,'\n average across p : %4.7f',pint0)
    fprintf(1,'\n simulation error (percent) : %4.7f',suminvlikeerror);
    
    Mh(L) = pint0;
    
      
end         % end lambdaseq
fprintf(1,'\n\n\n ');
fprintf(1,'\n lambda: %2.2f marg lik: %9.9f',[lambdaseq;Mh]);

fprintf(fid00,'\n lambda: %2.2f marg lik: %9.9f',[lambdaseq;Mh]);

fclose(fid00);		% close output file

%| PLOTTING
figure('Name', ['   No Simulations : ', num2str( nblocks*nsim ) ]);
title('Replicating DNS''s Results with *CFP* Codes')
xlabel('\lambda')
ylabel('Marginal Data Density : Gelfand-Dey Method - Geweke Correction')
% 1 % 
% plot(lambdaseq,Mh); grid on;
% hold on; plot(lambdaseq,Mh, 'o');

% 2 %
ChooseLambda    = [ 1 : size(lambdaseq,2) ];
lambdaseq_tmp   = lambdaseq( ChooseLambda );
Mh_tmp          = Mh( ChooseLambda );
plot(lambdaseq_tmp,Mh_tmp); grid on;
hold on; plot(lambdaseq_tmp,Mh_tmp, 'o');
