%! This function is related to " marglhnew.m " in the original DNS formulation
%! Modified to take FAVAR structure into account
function PY = CFP_MDD( para, yyyy, wwyy, wwww, nobs, lambda, nlags, nvar, mspec, npara, FFFFt);

    % PRIOR from Dummy OBS
        [TTT,RRR,valid] = dsgesolv(para,mspec);
        [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara);
        
    % POSTERIOR: MLE + DSGE + TRAINING SAMPLE        
		if eq( retcode, 0 )
			% inderterminacy case
			PY = -1E10;
		else
            %| FULL PRIOR
                m       = size( yyyy, 1);
                p       = nlags;
            if not( isempty( FFFFt ) )
                wwyyd   = zeros( size(wwyy) );
                wwwwd   = zeros( size(wwww) );
                wwwwd(1: 1 + m*p, 1: 1 + m*p )         = xxxxd;
                wwwwd(2 + m*p : end, 2 + m*p : end )   = FFFFt;
                wwyyd(1: 1 + m*p, : )                  = xxyyd;
                wwyyd(2 + m*p : end, : )               = zeros( size(FFFFt, 1), m );
            else
                wwwwd   = xxxxd;
                wwyyd   = xxyyd;
            end
            [k1, k2]= size( wwyyd);
            %keyboard
            %plot( [ reshape( wwww(14:25, 14:25), 144, 1 )/80, reshape( FFFFt, 144, 1 ) ] )
            
            %| POSTERIOR
                wwyy  = wwyy(1:k1, 1:k2);
                wwww  = wwww(1:k1, 1:k1);
			    yyyyc = yyyy + lambda*yyyyd;
			    wwyyc = wwyy + lambda*wwyyd;
			    wwwwc = wwww + lambda*wwwwd;
                
            %| DEGREES OF FREEDOM
			    T   = nobs;
                k   = size( wwyyc, 1 );
                vt  = lambda - k;
                vb  = T + lambda - k;
			
			%| VCV
			    Sig_hat     = ( yyyyc - wwyyc'*( wwwwc \ eye(k) )*wwyyc ) / (vb + k);
			    Sig_hatd    = ( yyyyd - wwyyd'*( wwwwd \ eye(k) )*wwyyd );
                
            %| MDD
                nobsc  = vb + k;                
				lnkapt = (-nvar*(nobs)/2)*log(2*pi) + ((nobs)*nvar/2)*log(2);
                for i = 1:nvar
                    lnkapt = lnkapt+gammaln((nobsc-k+1-i)/2)-gammaln((lambda-k+1-i)/2);
                end
				PYdns= lnkapt  - ((nobsc-k)/2) *log(det(nobsc *Sig_hat )) + ...
                                 ((lambda-k)/2)*log(det(lambda*Sig_hatd)) - ...
                                 (nvar/2)*log(det(wwwwc)/det(lambda*wwwwd));
                             
            %| MDD : My Way
            try
                PY  = (-m*T/2)*log(2*pi) + (m*T/2)*log(2) + MGammaLN( m, vb ) - MGammaLN(m, vt)  - ...
                                                             ( vb/2 ) * lndet( (vb+k)*Sig_hat  ) + ...
                                                             ( vt/2 ) * lndet( (vt+k)*Sig_hatd ) - ...
                                                   (m/2)*( lndet( wwwwc) - lndet(lambda*wwwwd) );
           catch
               keyboard
           end
                                                         
                %PY, PYdns

        end

        function x = dlyap(a,c)
%DLYAP	Discrete Lyapunov equation solver.
%
%    X = DLYAP(A,Q) solves the discrete Lyapunov equation:
%
%		A*X*A' - X + Q = 0
%
%    See also  LYAP.

%	J.N. Little 2-1-86, AFP 7-28-94
%	Copyright 1986-2002 The MathWorks, Inc. 
%	$Revision: 1.12 $  $Date: 2002/04/04 15:18:42 $

% How to prove the following conversion is true.  Re: show that if
%         (1) Ad X Ad' + Cd = X             Discrete lyaponuv eqn
%         (2) Ac = inv(Ad + I) (Ad - I)     From dlyap
%         (3) Cc = (I - Ac) Cd (I - Ac')/2  From dlyap
% Then
%         (4) Ac X + X Ac' + Cc = 0         Continuous lyapunov
% 
% Step 1) Substitute (2) into (3)
%         Use identity 2*inv(M+I) = I - inv(M+I)*(M-I) 
%                                 = I - (M-I)*inv(M+I) to show
%         (5) Cc = 4*inv(Ad + I)*Cd*inv(Ad' + I)
% Step 2) Substitute (2) and (5) into (4)
% Step 3) Replace (Ad - I) with (Ad + I -2I)
%         Replace (Ad' - I) with (Ad' + I -2I)
% Step 4) Multiply through and simplify to get
%         X -inv(Ad+I)*X -X*inv(Ad'+I) +inv(Ad+I)*Cd*inv(Ad'+I) = 0
% Step 5) Left multiply by (Ad + I) and right multiply by (Ad' + I)
% Step 6) Simplify to (1)

[m,n] = size(a);
a = (a+eye(m))\(a-eye(m));
c = (eye(m)-a)*c*(eye(m)-a')/2;
x = lyap(a,c);

% end dlyap
