% This file contains the specification of the VAR model
% OUTPUT
% mspec:        code of the DSGE model (<100 2 exogenous variables, >=100 3 exogenous variables)
% mprior:       code for the choice of the prior
% dataset:      code for the data set (( used in pathspec (floor(404.2)) ))
% dates:        end of the sample
% lambdaseq:    vector of values of lambda
% fisparamseq:  0/1 stochastic/fixed parameters
% new:          controls the type of priors ('new' is the default value)
% JustMode:     0/1 do not / compute forecast with simulations (at the mode only)
% qahead:       quarters ahead for computing forecasts

% Configure the Metropolis Algorithm (number of simulations)
% nblocks:      
% nsim:         nbolcks*nsim=total simulations
% nburn:        number of burn-in chain draws

% stime:        n. obs. to use in VAR
% st:           string = stime
% nlags:        number of lags
% T0:           last observation discarded
% Tend:         0 to the end of the data set
% npara:        number of parameters (vd. mspec)
% para_names:   string, names of the parameters (vd. mspec)
% bounds:       bounds of the parameter space
% para:         initial values of parameters (starting point for the MCMC posterior simulator)

% varnames:     names of shocks, and variables for which we want cumulative forecasts
% names_shocks: names of shocks, and variables for which we want cumulative irf
% cum_for:      [1 1]/[1 1 0] for mspec <100 / >=100
% cum_irf:      

% identify policy and non policy parameters
% polipar:      = 6, w/ mspec=101
% polivalue:    = 1, w/ mspec=101

% function [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;


%| Choose Model { CGG 2000 }
    mspec   = 101;
%| Choose Prior { File name entry in () : see -> ./prior/m101(2)pri.m }
    mprior  = 2; %% 2 is standard
%| Choose dataset 
    dataset = 404.2;
%| Choose date
    dates_dns = 2001.5;   
%| LAMBDA: Effective sample size to generate prior 
    % lambdaseq   = [0.2 .35 .4 0.6 .8  1  1.4  1.8  10 20  Inf];
    % lambdaseq   = [0 .20  .35  .40  .60  .80  .90  1.0  1.4  2.0  4.0  8.0  10 20];   % Replicating DNS
    % lambdaseq   = [0 .11 .15 .20 .25 .35 .40 .60 .80 .90  1.0  1.4  2.0  4.0  10 20]; % Replicating DNS { p = 1 }
    % lambdaseq   = [0 .2375  .35  .40  .60  .80  .90  1.0  1.4  2.0  4.0  8.0  10 20];   % DSGE-FAVAR(4,1)
    % lambdaseq   = [0 .35  .40  .60  .80  .90  1.0  1.4  2.0  4.0  8.0  10 20 Inf ];   % DSGE-FAVAR(4,4)
    
%  lambdaseq   = [0 .1 .15 .2  .40  .50 .60  .7  1.0  1.4  1.8  10 ]; 
    lambdaseq   = [0.6 0.8 1];
%    dispcat('Lambda_min =', (k + m) / T); disp(' ')
%| FIXparamseq { =1 exogenous (fixed parameters) at para, =0 otherwise }
    fixparamseq = [0];
%| "new" controls the type of prior (normalization), 'new' is the default: obfcnnew -> Optimization
    new         = 'new';
%| JustMode {=1 No simulation, Compute FCST at mode }
    JustMode    = 0;           
%| Quarters ahead for computing forecasts
    qahead      = H;
%| Metropolis Algorithm ( number of simulations )
    nblocks     =   10;
    nsim        = 1;
%     nsim=2500;
    nburn       = nsim;
%| Number of observations from MLE-VAR
    stime       = T;
    st          = num2str(stime);
%| Lags
    nlags       = p;
%| End of Presample
    T0          = nlags;
    Tend        = 0;        % End of Estimation Sample 
%| Endogenous variables 
    nvar        = 3;
    m           = 3;
%| Parameters in DSGE model
    npara       = 13;
%| Parameters in DSGE model: Labels 
    para_names = { 'lngamm';'lnpist';'lnrrst';'kap';'tau';'psi1';'psi2';'rhor';'rhog';'rhoz';'sigr';'sigg';'sigz'};
%| Bounds for MH Algorithm
    bounds = [...
    [-1,    10          ]; ...
    [-1,    10          ]; ...
    [1E-6,  10          ]; ...
    [-10,   10          ]; ...
    [1E-5,  100         ]; ...
    [1E-5,  10          ]; ...
    [1E-5,  10          ]; ...
    [1E-6,  0.9999999   ]; ...
    [1E-6,  0.9999999   ]; ...
    [1E-6,  0.9999999   ]; ...
    [1E-7,  500         ]; ...
    [1E-7,  500         ]; ...
    [1E-7,  500         ]];
%| Starting Values For Parameters { Optimization // Hessian }
    lngamm      = 0.5;
	lnpist      = 1;
	lnrrst      = 0.5;
	kap         = 0.4;
	tau         = 1;
	psi1        = 2.5;
	psi2        = 0.3;
	rhor        = 0.4;
	rhog        = 0.8;
	rhoz        = 0.2;
	sigr        = 0.5;
	sigg        = .5;
	sigz        =  1;
    
	para        = [ lngamm ; lnpist ; lnrrst ; kap ; tau ; psi1 ; psi2 ;...
                    rhor ; rhog ; rhoz ; sigr ; sigg ; sigz ];
                
%| Cumulative FCST & IRFs
    varnames    = {'Real GDP Growth'; 'Inflation Rate'; 'Fed Funds Rate' };
    cum_for     = [1 1 0];
    cum_irf     = [1];
    names_shocks= {'Money';'Gvmt';'Tech'};

%| Identify Policy and Non-Policy Parameters
    polipar     = [6];
    polivalue   = [1];