% filename: loaddat2.g
% description: reads time series data from ASCII data set into
% Matlab variables YYall, XXall
% ready for VAR estimation
% created:  05/03/00
% modified: 06/10/05

% Time series, extracted 21/11/01
% Gross Domestic Product (SAAR, Bil.Chn.1996$), Bureau of Economic Analysis
% CPI-U: All Items (SA, 1982-84=100), Bureau of Labor Statistics
% Federal Funds Rate 1,2
% Quarterly Data from 1947:I to 2001:III
function [YYall,XXall,ti,nobs] = loaddat2(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

datafile = [fpath '\data4.txt'];
data_q = importdata(datafile);
[nr_q,nc_q]=size(data_q);
% Compute growth rates: from 1947:II to 2001:III, 218 observations
series =  [100*( log(data_q(2:nr_q,1))-log(data_q(1:nr_q-1,1)) ) ...
              ( data_q(2:nr_q,2:3) )];
% Eliminate Observations for which the Federal Funds Rate is not available
% -> 189 observations, starting 1954:III
series        = series(30:size(series,1),:);
[nobs,nc_s]   = size(series);
ti            = [1954.5:0.25:1954.5+0.25*(nobs-1)]';

% Note: we need at least T0 = nlags to initialize the lags of the VAR
YYall     = series(1+T0:nobs,:);
XXall     = ones(nobs-T0, 1+nvar*nlags);
ti        = ti(1+T0:nobs,:);

i = 1;
while i <= nlags
   XXall(:,2+(i-1)*nvar:1+i*nvar) = series(T0-i+1:nobs-i,:);
   i = i+1;
end

YY     = YYall(1:Tend-T0,:);
XX     = XXall(1:Tend-T0,:);
if Tend-T0+1 <= size(YYall,1)
   YYfut  = YYall(Tend-T0+1:size(YYall,1),:);
else
   YYfut  = zeros(1,nvar);
end

XXpred = [1, YYall(Tend-T0,:)];
if nlags > 1
    XXpred = [XXpred, XXall(Tend-T0,2:1+nvar*(nlags-1))];
end

nobs = size(YY,1);
nobsfut = min([89;size(YYfut,1)]);
YYfut = YYfut(1:nobsfut,:);

%**********************************************
%           Specify Minnesota Prior
%**********************************************

% Prior for Sigma
nu_min = 3;    
H_min = nu_min*eye(nvar);

% Prior for C
CClag1 = zeros(nvar,nvar);
CClag1(3,3) = 1;
if nlags == 1
   CCt_min = [zeros(nvar,1), CClag1];
else
   CCt_min = [zeros(nvar,1), CClag1, zeros(nvar,nvar*(nlags-1))];
end
C_min   = vec(CCt_min');

% run univariate AR(1)'s
ar1_sig = zeros(nvar,1);
i = 1;
while i <= nvar
   yyar1 = YY(2:nobs,i);
   xxar1 = [ones(nobs-1,1), YY(1:nobs-1,i)];
   ar1_sig(i) = (yyar1'*yyar1 - yyar1'*xxar1*((xxar1'*xxar1)\(xxar1'*yyar1)))/(nobs-1);
   i = i+1;
end

% Prior variance for intercept 
intcpt_v0 = 5; 
% Prior variance for autoregressive
% coefficients
SIG   = kron(ones(nvar,1),ar1_sig');
SIG   = SIG.*(100*ones(3,3)-diag(75*ones(3,1)));

VC_mininv  = ones(nvar,1)/intcpt_v0;
i = 1;
while i<= nlags
   VC_mininv = [VC_mininv,SIG.*i^2];
   i = i+1;
end
VC_mininv_ = diag(vec(VC_mininv'));
VC_mininv = eye(nvar*(1+nvar*nlags)).*(vec(VC_mininv')*ones(1,nvar*(1+nvar*nlags)));
diag(VC_mininv_-VC_mininv)