% Horizontal Concatenation of Lagged Vector-Valued Variables
% 
% Example : 1) y = rand(6,2),  [ y, GNLag(y,1), GNLag(y,2)]
%           2) [Ylag, y] = LagByLags(y, 2);  y, Ylag
%
%
% Agostino  Oct 2005

function [Ylag, y] = LagByLags(y, nlag)
    
    Ylag = [];
    for i = 1 : nlag
        Ylag = [Ylag, GNLag(y, i) ];
    end
    
    Ylag(1:nlag, :) = [];
    y(1:nlag, :)    = []; 

% ADDITIONAL FUNCTION ::
    function M = GNLag(X, p)
        
        [T, m] = size(X);
        M      = GNaN(T,m);
        
        if ge(p, T)
            warning('> GNLag : Insufficient Obs LAGS >= T ')
        end
        if gt(m, T)
            warning('> GNLag : It assumed to work with T > m')
        end
        
        M( p + 1 : end, : ) = X( 1 : end - p, : );