function [xmin,fbrent] = brent(ax,bx,cx,tol,pcom,xicom,func,varargin);
% take initial brackets as inputs (ax,bx,cx, with ax<bx<cx and func(bx)<func(ax) , func(bx)<func(cx))
% isolates the minimum uisng brent method. returns the abscissa of the minimum (xmin) and the corresponding function value (fbrent)
% taken from fortran routine with the same name (see Numerical Recipes by Press et al...)

% parameters
ITMAX = 100;
CGOLD = .3819660;
ZEPS = 1e-10;

% ax and cx must be in ascending order. if not, reorganize
% works for scalars only
if size(ax,1) == 1
    a = min(ax,cx);
    b = max(ax,cx);     
else
    a = ax;
    b = bx;
end

% Initialize
v = bx;
clear ax bx cx;

w = v;
x = v;
e = 0; % zeros(size(ax));
fx = f1dim(x,pcom,xicom,func,varargin{:});
fv = fx;
fw = fx;

for iter = 1:ITMAX;
    xm = .5*(a+b);
    tol1 = tol*abs(x)+ZEPS;
    tol2 = 2*tol1;
    if abs(x-xm) <= (tol2-.5*(b-a)) % test for done
        xmin = x;
        fbrent = fx;
        return
    end
    if abs(e) > tol1 %try parabolic fit
        r = (x-w)*(fx-fv);
        q = (x-v)*(fx-fw);
        p = (x-v)*q-(x-w)*r;
        q = 2*(q-r);
        if q > 0 
            p = -p; 
        end
        q = abs(q);
        etemp = e;
        e = d;
        if ( ( abs(p) >= abs(.5*q*etemp) ) | ( p <= q*(a-x)) | (p >= q*(b-x)) ) % if any of this condition holds, go to Golden search
            if x >= xm
                e = a-x;
            else
                e = b-x;
            end         
            d = CGOLD*e;            
            
        else % take parabolic step
            d = p/q;
            u = x+d;
            if ( ((u-a) < tol2) | (b-u < tol2));
                d = sign(xm-x)*tol1;
            end
        end
    else % go straight to Golden search
        if x >= xm
            e = a-x;
        else
            e = b-x;
        end         
        d = CGOLD*e;                   
    end
                
    % get here with d computed from either parabolic search or golden search
    if abs(d) >= tol1
        u = x+d;
    else
        u = x+sign(d)*tol1;
    end
    % evaluate function
    fu = f1dim(u,pcom,xicom,func,varargin{:});
    if fu <= fx
        if u >= x
            a = x;
        else
            b = x;
        end
        v = w;
        fv = fw;
        w = x;
        fw = fx;
        x = u;
        fx = fu;
    else
        if u < x
            a = u;
        else
            b = u;
        end
        if ( (fu <= fw) | (w == x) )
            v = w;
            fv = fw;
            w = u;
            fw = fu;
        elseif ( (fu <= fv) | (v == x) | (v == w) )
            v = u;
            fv = fu;
        end
    end
end

disp('brent exceeded maximum number of iterations');

xmin = x;
fbrent = fx;

