function [fh,xh,gh,H,itct,fcount,retcodeh] = csmindel(fcn,x0,H0,grad,crit,nit,Verbose,varargin)
%[fhat,xhat,ghat,Hhat,itct,fcount,retcodehat] = csminwel(fcn,x0,H0,grad,crit,nit,varargin)
% fcn:   string naming the objective function to be minimized
% x0:    initial value of the parameter vector
% H0:    initial value for the inverse Hessian.  Must be positive definite.
% grad:  Either a string naming a function that calculates the gradient, or the null matrix.
%        If it's null, the program calculates a numerical gradient.  In this case fcn must
%        be written so that it can take a matrix argument and produce a row vector of values.
% crit:  Convergence criterion.  Iteration will cease when it proves impossible to improve the
%        function value by more than crit.
% nit:   Maximum number of iterations.
% varargin: A list of optional length of additional parameters that get handed off to fcn each
%        time it is called.
%        Note that if the program ends abnormally, it is possible to retrieve the current x,
%        f, and H from the files g1.mat and H.mat that are written at each iteration and at each
%        hessian update, respectively.  (When the routine hits certain kinds of difficulty, it
%        write g2.mat and g3.mat as well.  If all were written at about the same time, any of them
%        may be a decent starting point.  One can also start from the one with best function value.)
%        original program by Chris Sims/Jinill Kim, modified by Marco Del Negro on 09/01

[nx,no]=size(x0);
nx=max(nx,no);
%%% Recompute H numerically at the beginning of the program 
RECOMPUTEH = 0;
%%% Updates H numerically if bfgsi fails
UPDATEH = 0;
%%% tries Fletcher-Reeves
TRYFR = 1;
%%% tries Powell
TRYPOWELL = 1;
%%% tries line minimization when csminit fails
TRYLINMIN = 1;

if Verbose
    fprintf(1,'\n Recompute H numerically at the beginning of the program: %1.1g',RECOMPUTEH);
    fprintf(1,'\n Updates H numerically if bfgsi fails: %1.1g ',UPDATEH);
    fprintf(1,'\n tries Fletcher-Reeves: %1.1g ',TRYFR);
    fprintf(1,'\n tries Powell: %1.1g ',TRYPOWELL);
    fprintf(1,'\n tries line minimization when csminit fails: %1.1g \n',TRYLINMIN);
end
NumGrad= ( ~isstr(grad) | length(grad)==0);
				 	%%%%NumGrad=0 if grad is a non-empty string
done=0;
itct=0;
fcount=0;
snit=100;
%tailstr = ')';
%stailstr = [];
% Lines below make the number of Pi's optional.  This is inefficient, though, and precludes
% use of the matlab compiler.  Without them, we use feval and the number of Pi's must be
% changed with the editor for each application.  Places where this is required are marked
% with ARGLIST comments
%for i=nargin-6:-1:1
%   tailstr=[ ',P' num2str(i)  tailstr];
%   stailstr=[' P' num2str(i) stailstr];
%end
f0 = eval([fcn '(x0,varargin{:})']);
%ARGLIST
%f0 = feval(fcn,x0,P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,P11,P12,P13);
% disp('first fcn in csminwel.m ----------------') % Jinill on 9/5/95
if f0 > 1e50, disp('Bad initial parameter.'), return, end
if NumGrad		
				 	%%%%if grad is not string, either compute it (if grad=[]) or check whether it is fine)
   if length(grad)==0
      [g badg] = numgrad(fcn,x0, varargin{:});
      %ARGLIST
      %[g badg] = numgrad(fcn,x0,P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,P11,P12,P13);
   else
      badg=any(find(grad==0));
      g=grad;
   end
   %numgrad(fcn,x0,P1,P2,P3,P4);
else
				 	%%%%if grad is a string, compute it 
   [g badg] = eval([grad '(x0,varargin{:})']);
   %ARGLIST
   %[g badg] = feval(grad,x0,P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,P11,P12,P13);
   
   if RECOMPUTEH
        disp('compute variance')
          tic
          [H0 badH] = feval('numgradvec2',grad,x0,varargin{:});
          toc
          
          H0 = .5*H0+.5*H0';
          [Ve,Di]=eig(H0);              %[Ve,Di]=eig(V,'nobalance');
          Di = diag(Di);
          g = g.*sign(Di);
          Di = Di.*sign(Di);
          Di(find(Di)<1e-8) = 1e-8;
          Di(find(Di)>1e8) = 1e8;
          H0 = Ve*diag(1./Di)*Ve';
          clear Ve Di;

    end
end
retcode3=101;
x=x0;
f=f0;
H=H0;
cliff=0;
while ~done
					%%%% iterate until done is positive -while done is zero
   g1=[]; g2=[]; g3=[];
   %addition fj. 7/6/94 for control
   disp('-----------------')
   disp('-----------------')
   %disp('f and x at the beginning of new iteration')
   disp(sprintf('f at the beginning of new iteration, %20.10f',f))
   %-----------Comment out this line if the x vector is long----------------
   %   disp([sprintf('x = ') sprintf('%15.8g',x)]);
   %-------------------------
   itct=itct+1;
   
   [f1 x1 fc retcode1] = csminit(fcn,x,f,g,badg,H,Verbose,varargin{:});
   %ARGLIST
   %[f1 x1 fc retcode1] = csminit(fcn,x,f,g,badg,H,P1,P2,P3,P4,P5,P6,P7,...
   %           P8,P9,P10,P11,P12,P13);
   % itct=itct+1;
   fcount = fcount+fc;
   % erased on 8/4/94
   % if (retcode == 1) | (abs(f1-f) < crit)
   %    done=1;
   % end
   % if itct > nit
   %    done = 1;
   %    retcode = -retcode;
   % end

   if retcode1 ~= 1
					%%%% if retcode1=1 gradient is zero and you are at the peak                             
          
      if retcode1==2 | retcode1==4
          
					%%%% if retcode1=2 or 4 you have shrunk or increased lambda as much as you could: exhausted search possibilities
					%%%% the csminit step has failed
          if ~NumGrad & UPDATEH              
              disp('Cliff. Try to recompute H')
              tic
              [Hnew badH] = feval('numgradvec2',grad,x,varargin{:});
              toc
              %%% compute eigenvalues & eigenvectors
              Hcliff = .5*Hnew+.5*Hnew';
              clear Hnew;
              [Ve,Di]=eig(Hcliff);              %[Ve,Di]=eig(V,'nobalance');
              Di = diag(Di);
              g = g.*sign(Di);
              Di = Di.*sign(Di);
              Di(find(Di)<1e-8) = 1e-8;
              Di(find(Di)>1e8) = 1e8;
              Hcliff = Ve*diag(1./Di)*Ve';
              clear Ve Di;
              [f11 x11 fc retcode11] = csminit(fcn,x,f,g,badg,Hcliff,Verbose,varargin{:});  
              if retcode11 == 2 | retcode11 == 4 | retcode11 == 6
                  wall1=1; badg1=1;            
              else
                  H = Hcliff;
                  x1 = x11;
                  f1 = f11;
                  retcode1 = retcode11;
		
                  if NumGrad
                      [g1 badg1] = numgrad(fcn, x1,varargin{:});
                     %ARGLIST
                     %[g1 badg1] = numgrad(fcn, x1,P1,P2,P3,P4,P5,P6,P7,P8,P9,...
                     %                P10,P11,P12,P13);
                  else
                     [g1 badg1] = eval([grad '(x1,varargin{:})']);
                     %ARGLIST
                     %[g1 badg1] = feval(grad, x1,P1,P2,P3,P4,P5,P6,P7,P8,P9,...
                     %                P10,P11,P12,P13);
                  end
                  wall1=badg1;
                  % g1
                  save g1 g1 x1 f1 varargin;
                   
              end
          else
              wall1 =1; badg1 = 1;
          end
       
      else
         if retcode1 == 6 % if csminit did not improve            
            if TRYLINMIN
        		if Verbose; disp(sprintf('no improvement from csmint (retcode1 = %8.1f); try line minimization',retcode1)); end;
        		[x11,f11] = linmin(x,-H0*g,fcn,varargin{:});
			    if ( (abs(f11-f) > crit) & (f11 < f1))
                  if Verbose; disp(sprintf('line minimization worked, improvement: %18.9f',abs(f11-f))); end;                      
                  x1 = x11; clear x11;
                  f1 = f11; clear f11;
                  retcode1  = 8;
                end
            end
         end
					%%%% if you are not at the peak but the csminit has improved, compute the gradient again to update H
         if NumGrad
            [g1 badg1] = numgrad(fcn, x1,varargin{:});
            %ARGLIST
            %[g1 badg1] = numgrad(fcn, x1,P1,P2,P3,P4,P5,P6,P7,P8,P9,...
            %                P10,P11,P12,P13);
         else
            [g1 badg1] = eval([grad '(x1,varargin{:})']);
            %ARGLIST
            %[g1 badg1] = feval(grad, x1,P1,P2,P3,P4,P5,P6,P7,P8,P9,...
            %                P10,P11,P12,P13);
         end
         wall1=badg1;
         % g1
         save g1 g1 x1 f1 varargin;
         %ARGLIST
         %save g1 g1 x1 f1 P1 P2 P3 P4 P5 P6 P7 P8 P9 P10 P11 P12 P13;
      end
      if wall1 % & (~done) by Jinill
         % Bad gradient or back and forth on step length.  Possibly at
         % cliff edge.  Try changing search direction by recomputing H
         %
         disp('Cliff.  Perturbing search direction.')
         Hcliff=H+diag(diag(H).*rand(nx,1));
         
         [f2 x2 fc retcode2] = csminit(fcn,x,f,g,badg,Hcliff,Verbose,varargin{:});
         %ARGLIST
         %[f2 x2 fc retcode2] = csminit(fcn,x,f,g,badg,Hcliff,P1,P2,P3,P4,...
         %     P5,P6,P7,P8,P9,P10,P11,P12,P13);
         fcount = fcount+fc; % put by Jinill

         if retcode2 == 6

            if TRYLINMIN
                if Verbose; disp(sprintf('no improvement in second step (retcode2 = %18.1f); try line minimization',retcode2)); end;
                [x21,f21] = linmin(x,-Hcliff*g,fcn,varargin{:});
            else
                x21 = x2; f21 = f2;
            end
            
		    if ( (abs(f21-f) > crit) & (f21 < f2)) 
                if Verbose; disp(sprintf('line minimization worked, improvement: %18.9f',abs(f21-f))); end;
                x2 = x21; clear x21;
                f2 = f21; clear f21;
                retcode2  = 8;
                g2 = [];
                wall2=1; badg2 = 1;
                %%% if found a good direction, try transversing
            else               
                if Verbose; disp('try Powell '); end;
                xi = eye(length(x));
                [xp2,fp2] = powell(x,xi,crit,2,fcn,varargin{:});
                    
                if (abs(fp2-f) > crit) & (fp2 < f)
                    x2 = xp2;
                    f2 = fp2;
                    g2 = [];
                    if Verbose; disp(sprintf('Powell worked, improvement: %18.9f',abs(f2-f))); end                        
                    wall2=1; badg2 = 1; 
                    %%% if found a good direction, try transversing
                end
                clear xp2 fp2;
            end                         
         end
         
         if  f2 < f
            if retcode2==2 | retcode2==4
                %%% if improvement in csminit but cliff, try powell to find a better direction -> either way, try transversing                 if Verbose; disp('try Powell '); end;
                xi = eye(length(x));
                [xp2,fp2] = powell(x,xi,crit,2,fcn,varargin{:});
                    
                if (abs(fp2-f) > crit)
                    x2 = xp2;
                    f2 = fp2;
                    g2 = [];
                    if Verbose; disp(sprintf('Powell worked, improvement: %18.9f',abs(f2-f))); end      
                end
                clear xp2 fp2;

                wall2=1; badg2 = 1;
            elseif retcode2 == 6
                %%% if no improvement in csminit, powell already tried -> go to 3
            else
                %%% step 2 worked fine, compute gradient and skip step 3
                %%% change H to Hcliff
                H = Hcliff;
              
                if NumGrad
                   [g2 badg2] = numgrad(fcn, x2,varargin{:});
                   %ARGLIST
                   %[g2 badg2] = numgrad(fcn, x2,P1,P2,P3,P4,P5,P6,P7,P8,...
                   %      P9,P10,P11,P12,P13);
                else
                   [g2 badg2] = eval([grad '(x2,varargin{:})']);
                   %ARGLIST
                   %[g2 badg2] = feval(grad,x2,P1,P2,P3,P4,P5,P6,P7,P8,...
                   %      P9,P10,P11,P12,P13);
                end
                wall2=badg2;
                % g2
                save g2 g2 x2 f2 varargin
                %ARGLIST
                %save g2 g2 x2 f2 P1 P2 P3 P4 P5 P6 P7 P8 P9 P10 P11 P12 P13;
            end
            if wall2
               disp('Cliff again.  Try traversing')
               if norm(x2-x1) < 1e-13
                  f3=f; x3=x; badg3=1;retcode3=101;
               else
                  gcliff=((f2-f1)/((norm(x2-x1))^2))*(x2-x1);
                  [f3 x3 fc retcode3] = csminit(fcn,x,f,gcliff,0,eye(nx),Verbose,varargin{:});
                  %ARGLIST
                  %[f3 x3 fc retcode3] = csminit(fcn,x,f,gcliff,0,eye(nx),P1,P2,P3,...
                  %         P4,P5,P6,P7,P8,...
                  %      P9,P10,P11,P12,P13);
                  fcount = fcount+fc; % put by Jinill
                  if retcode3 == 6 | retcode3==2 | retcode3==4
                        %%% if no improvement in csminit, try line min 
 
                        if TRYLINMIN
                       		if Verbose; disp(sprintf('no improvement from csmint (retcode3 = %8.1f); try line minimization',retcode3)); end;
                    		[x31,f31]  = linmin(x,-gcliff,fcn,varargin{:});
				            if ( (abs(f31-f) > crit) & (f31 < f3) )
                               if Verbose; disp(sprintf('line minimization worked, improvement: %18.9f',abs(f31-f))); end;                       
                               x3 = x31; clear x31;
                               f3 = f31; clear f31;
                               retcode3  = 8;
                               g3 = [];
                            end
                        end
                        wall3 = 1; badg3 = 1;
                  else
                         if NumGrad
                            [g3 badg3] = numgrad(fcn, x3,varargin{:});
                            %ARGLIST
                            %[g3 badg3] = numgrad(fcn, x3,P1,P2,P3,P4,P5,P6,P7,P8,...
                            %                        P9,P10,P11,P12,P13);
                         else
                            [g3 badg3] = eval([grad '(x3,varargin{:})']);
                            %ARGLIST
                            %[g3 badg3] = feval(grad,x3,P1,P2,P3,P4,P5,P6,P7,P8,...
                            %                         P9,P10,P11,P12,P13);
                         end
                         wall3=badg3;
                         % g3
                         save g3 g3 x3 f3 varargin;
                         %ARGLIST
                         %save g3 g3 x3 f3 P1 P2 P3 P4 P5 P6 P7 P8 P9 P10 P11 P12 P13;
                  end
               end
            else
               f3=f; x3=x; badg3=1; retcode3=101;
            end
         else
            f3=f; x3=x; badg3=1;retcode3=101;
         end
      else
         % normal iteration, no walls, or else we're finished here.
         f2=f; f3=f; badg2=1; badg3=1; retcode2=101; retcode3=101;
      end
   end
   %how to pick gh and xh
   if f3<f & badg3==0
					%%%% if 3 (transversing) was needed, it improved and gradient is good, take that
      ih=3
      fh=f3;xh=x3;gh=g3;badgh=badg3;retcodeh=retcode3;
   elseif f2<f & badg2==0
					%%%% if 2 (perturbig) was needed, it improved and gradient is good, take that
      ih=2
      fh=f2;xh=x2;gh=g2;badgh=badg2;retcodeh=retcode2;
   elseif f1<f & badg1==0
					%%%% if first try went fine, take that
      ih=1
      fh=f1;xh=x1;gh=g1;badgh=badg1;retcodeh=retcode1;
   else
					%%%% if nothing worked, just take the min of your attempts and compute the gradient
      [fh,ih] = min([f1,f2,f3]);
      disp(sprintf('ih = %d',ih))
      %eval(['xh=x' num2str(ih) ';'])
      switch ih
         case 1
            xh=x1;
         case 2
            xh=x2;
         case 3
            xh=x3;
      end %case
      %eval(['gh=g' num2str(ih) ';'])
      %eval(['retcodeh=retcode' num2str(ih) ';'])
      retcodei=[retcode1,retcode2,retcode3];
      retcodeh=retcodei(ih);
      if exist('gh')
         nogh=isempty(gh);
      else
         nogh=1;
      end
      if nogh
         if NumGrad
            [gh badgh] = feval('numgrad',fcn,xh,varargin{:});
         else
            [gh badgh] =  eval([grad '(xh,varargin{:})']);
%feval('grad',xh,varargin{:});
         end
      end
      badgh=1;
   end
   %end of picking
 
   %ih
   %fh
   %xh
   %gh
   %badgh
   stuck = (abs(fh-f) < crit);
				%%%% if nothing REALLY worked, too bad, you're stuck
                if stuck & ~NumGrad & TRYFR
                    disp('try Fletcher-Reevs/Polak-Ribiere ');
                    [xh2,fh2] = frprmn(xh,crit,fcn,grad,varargin{:});
                    
                    if (abs(fh2-f) > crit)
                        xh = xh2;
                        fh = fh2;
                        stuck = 0;
                        retcodeh = 9;
                        disp(sprintf('\n Fletcher-Reeves/Polak-Ribiere worked, improvement: %18.9f',abs(fh2-f)));                        
                    end
                    clear xh2 fh2;
                end

                if stuck & TRYPOWELL
                    disp('try Powell ');
                    xi = eye(length(xh));
                    [xh2,fh2] = powell(xh,xi,crit,2,fcn,varargin{:});
                   
                    if (abs(fh2-f) > crit)
                        xh = xh2;
                        fh = fh2;
                        stuck = 0;
                        retcodeh = 10;
                        disp(sprintf('\n Powell worked, improvement: %18.9f',abs(fh2-f)));                        
                    end
                    clear xh2 fh2;
                end
   if  ( retcodeh == 9 | retcodeh == 10) 
         if NumGrad
            [gh badgh] = feval('numgrad',fcn,xh,varargin{:});
         else
            [gh badgh] =  eval([grad '(xh,varargin{:})']);
            %feval('grad',xh,varargin{:});
         end
   end

   if (~badg)&(~badgh)&(~stuck)
				%%%% if you are not stuck, and current and pervious gradients are OK, update H matrix
      H = bfgsi(H,gh-g,xh-x);
                
      if ( (retcodeh == 7) & UPDATEH)
          disp('bfgs did not work: recompute variance')
          tic
          [H badH] = feval('numgradvec2',grad,x0,varargin{:});
          toc
          
          H = .5*H+.5*H';
          [Ve,Di]=eig(H);              %[Ve,Di]=eig(V,'nobalance');
          Di = diag(Di);
          g = g.*sign(Di);
          Di = Di.*sign(Di);
          Di(find(Di)<1e-8) = 1e-8;
          Di(find(Di)>1e8) = 1e8;
          H = Ve*diag(1./Di)*Ve';
          clear Ve Di;
          
      end
   end
   if Verbose
      disp('----')
      disp(sprintf('Improvement on iteration %d = %18.9f',itct,f-fh))
   end
   if Verbose
      if itct > nit
         disp('iteration count termination')
         done = 1;
      elseif stuck
         disp('improvement < crit termination')
         done = 1;
      end
      rc=retcodeh;
      if rc == 1
         disp('zero gradient')
      elseif rc == 6
         disp('smallest step still improving too slow, reversed gradient')
      elseif rc == 5
         disp('largest step still improving too fast')
      elseif (rc == 4) | (rc==2)
         disp('back and forth on step length never finished')
      elseif rc == 3
         disp('smallest step still improving too slow')
      elseif rc == 7
         disp('warning: possible inaccuracy in H matrix')
      elseif rc == 8
         disp('csminit fails, line minimization succeeds, DelNegro saved the day')
%pause
      end
   end
   f=fh;
%   disp(sprintf('max percentage change in the parameters, %20.10f',100*max(log(abs(xh(find(xh))))-log(abs(x(find(xh)))))))
% put criterion found in Judd's book
   x=xh;
   g=gh;
   badg=badgh;
  
end
% what about making an m-file of 10 lines including numgrad.m
% since it appears three times in csminwel.m
