%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                SOLVING THE DSGE MODELS                 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% MSEL = 1 is standard Cash-in-Advance
%% MSEL = 2 is CIA with liquidity effect
%% MSEL = 3 is CIA with liquidity effect and feedback rule

function [TTT,RRR,valid] = dsgesolv(para,mspec);
%% this procedure computes the solution to the CIA models


valid = 1;
TTT = 0;
RRR = 0;

if mspec == 2
	
	alp = para(1);
	bet = para(2);
	gam = para(3);
	mst = exp(para(4));
	rho = para(5);
	psi = para(6);
	del = para(7);
	sig_eps = para(8)^2;
	sig_eta = para(9)^2;
	alp1 = para(10);    

    
elseif ( mspec >= 3 ) & ( mspec <= 6 )
	%% Define the model parameters
	%% MODIFICATION: para[4] is now inflation rate
	alp = para(1);
	bet = para(2);
	gam = para(3);
	mst = exp(para(3)+para(4));
	rho = para(5);
	psi = para(6);
	del = para(7);
	sig_eps = para(8)^2;
	sig_eta = para(9)^2;
	kap = para(10);
	phi1 = para(11);
	phi2 = para(12);

elseif ( mspec == 101 ) | ( mspec == 102 )

   lngamm  = para(1)/100;
   lnpist  = para(2)/100;
   lnrrst  = para(3)/100;
   kap   = para(4);
   tau   = para(5);
   psi1  = para(6);
   psi2  = para(7);
   rhor  = para(8);
   rhog  = para(9);
   rhoz  = para(10);
    
   gam = exp(lngamm);   
   bet = gam/exp(lnrrst);  %%% CHANGE
end

%% Determine the steady state of the stochastically
%% detrended model
if mspec < 100
    %% all CIA model have the same st. st.
	gst  = exp(-gam);
	
	if 1-gst*bet*(1-del) < 0
       valid = -1;
       return
	end
	
	khst = ( (1-gst*bet*(1-del)) / (alp*gst^alp*bet) )^(1/(alp-1));
	xist = ( ((khst*gst)^alp - (1-gst*(1-del))*khst)/mst )^(-1);
	nust = psi*mst^2/( (1-alp)*(1-psi)*bet*gst^alp*khst^alp );
	hst  = xist/(nust+xist);
	pst  = xist + nust;
	kst  = khst*hst;
	
	
	if hst == 1;
       valid = -2;
       return
	end
	
	lst  = psi*mst*hst/( (1-psi)*(1-hst) );
	cst  = mst/pst;
	dst  = lst - mst + 1;
	yst  = kst^alp*hst^(1-alp)*gst^alp;
	rst  = mst/bet;
	wst  = lst/hst;
	ist  = yst-cst;
	qst  = 1 - dst;
	
	kap1 = 2*kap*wst*mst^2/qst;
	kap2 = - 2*(1+bet)*kap*wst*mst^2/qst;
	kap3 = 2*bet*kap*wst*mst^2/qst;
	
	if (ist < 0) | (dst < 0) | ( qst < 0)
       valid = -2;
       return
	end
    
end


if mspec == 2;

    %% Portfolio Adjustment cost model with money growth rule

	%% Define the vector of "state"-variables
	%% (dm(t), dp(t), dk(t+1), dh(t), dy(t), dd(t+1), dq(t+1), dqt(t+1), dqb(t+1))
    nstate = 9;
	
	
	%% Define Matrices of canonical system  
	
	
  GAM0 = [[1         , 0  , 0   , 0            , 0 ,   0     , 0 , 0 , 0]; ...
         [mst/lst   , -1 , 0   , 0            , -1,   0     , 0 , 0 , 1]; ...
         [1-mst/lst , 0  , 0   , (1/(1-hst))  , 0 ,   0     , 0 , 0 , 0]; ...
         [cst       , -cst, kst , -(1-alp)*yst , 0 ,  0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , -(1-alp)     , 1 ,   0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , 0            , 0 ,   0     , 0 , 1 , 0]; ...
         [0 , 1 , 0 , (1-alp)                       , 0 , 0 , kap3 , 0 , 0]; ...
         [0 , 1 , 0 , (1-alp)*(1 - (1-del)*gst*bet) , 0 , 0           , 0    , 0 , 0]; ...
         [0 , 0 , 0 , 0                             , 0 , 1           , 0    , 0 , 0]];


  GAM1 = [[rho , 0 , 0       , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , qst/lst  , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , -qst/lst , 0 , 0 , 0]; ...
         [0   , 0 , kst/bet , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , alp     , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , 1        , 0 , 0 , 0]; ...
         [ (rho*mst/lst + rho^2 - (kap1+kap2*rho+kap3*rho^2)) , 0 , -alp , 0 , 0 , -qst/lst -(kap1-kap2), -(kap2-kap3) , kap1 , 0 ]; ...
         [ (rho^2-1)  , 1 , (1-alp)*(1-(1-del)*gst*bet)    , 0 , 0 , 0 , 0 , 0 , 0]; ...
         [ 0          , 0 , 0                              , 0 , 0 , 0 , 1 , 0 , 0]];
         
         
    C = zeros(nstate,1);	
	
            	
	PPSI  = [[ 0 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-kst/bet , 0 ]; ...         
          [-alp     , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]];
  
    PPI = [zeros(6,3);eye(3)];
    
elseif mspec == 3;
    %% Interest Feedback Rule w/ output

    %% Define the vector of "state"-variables
    %% [dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    nstate = 9;

	%% Define Matrices of canonical system  
	%% 
	%% interest-hours rule:  ( 0 ~ -(1-rho)*phi2 ~ 0   ~ -(1-rho)*(1-alp)*phi1 ~ 0 ~   0     ~ 0 ~ 0 ~ 1)|
	%% interest-outout rule: ( 0 ~ -(1-rho)*phi2 ~ 0   ~ 0 ~ -(1-rho)*phi1 ~   0     ~ 0 ~ 0 ~ 1)|
	%% for money rules: phi = - phi

    GAM0 = [[0  , -(1-rho)*phi2 , 0   , 0 , -(1-rho)*phi1,   0     , 0 , 0 , 1]; ...            
           [mst/lst   , -1 , 0   , 0            , -1,   0     , 0 , 0 , 1]; ...
         [1-mst/lst , 0  , 0   , (1/(1-hst))  , 0 ,   0     , 0 , 0 , 0]; ...
         [cst       , -cst, kst , -(1-alp)*yst , 0 ,  0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , -(1-alp)     , 1 ,   0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , 0            , 0 ,   0     , 0 , 1 , 0]; ...
         [0 , 1 , 0 , (1-alp)                       , 0 , 0 , kap3 , 0 , 0]; ...
         [0 , 1 , 0 , (1-alp)*(1 - (1-del)*gst*bet) , 0 , 0           , 0    , 0 , 0]; ...
         [0 , 0 , 0 , 0                             , 0 , 1           , 0    , 0 , 0]];

    GAM1 = [[(1-rho)*phi2, -(1-rho)*phi2 , 0       , 0 , 0 , 0        , 0 , 0 , rho]; ...
         [0   , 0 , 0       , 0 , 0 , qst/lst  , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , -qst/lst , 0 , 0 , 0]; ...
         [0   , 0 , kst/bet , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , alp     , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , 1        , 0 , 0 , 0];...
         [ (rho*mst/lst+rho^2-(kap1+kap2*rho+kap3*rho^2)) , 0 , -alp , 0 , 0 , -qst/lst-(kap1-kap2), -(kap2-kap3) , kap1 , 0]; ...         
         [ (rho^2-1)  , 1 , (1-alp)*(1-(1-del)*gst*bet)    , 0 , 0 , 0 , 0 , 0 , 0]; ...
         [ 0          , 0 , 0                              , 0 , 0 , 0 , 1 , 0 , 0]];

    C = zeros(nstate,1);
            	
	PPSI  = [[ -(1-rho)*phi2 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-kst/bet , 0 ]; ...         
          [-alp     , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]];
  
    PPI = [zeros(6,3);eye(3)];
    
elseif mspec == 4;
    %% Interest Feedback Rule w/ hrs

    %% Define the vector of "state"-variables
    %% [dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    nstate = 9;

	%% Define Matrices of canonical system  
	%% 
	%% interest-hours rule:  ( 0 ~ -(1-rho)*phi2 ~ 0   ~ -(1-rho)*(1-alp)*phi1 ~ 0 ~   0     ~ 0 ~ 0 ~ 1)|
	%% interest-outout rule: ( 0 ~ -(1-rho)*phi2 ~ 0   ~ 0 ~ -(1-rho)*phi1 ~   0     ~ 0 ~ 0 ~ 1)|
	%% for money rules: phi = - phi

    GAM0 = [[0  , -(1-rho)*phi2 , 0   , -(1-rho)*(1-alp)*phi1, 0,   0     , 0 , 0 , 1]; ...            
           [mst/lst   , -1 , 0   , 0            , -1,   0     , 0 , 0 , 1]; ...
         [1-mst/lst , 0  , 0   , (1/(1-hst))  , 0 ,   0     , 0 , 0 , 0]; ...
         [cst       , -cst, kst , -(1-alp)*yst , 0 ,  0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , -(1-alp)     , 1 ,   0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , 0            , 0 ,   0     , 0 , 1 , 0]; ...
         [0 , 1 , 0 , (1-alp)                       , 0 , 0 , kap3 , 0 , 0]; ...
         [0 , 1 , 0 , (1-alp)*(1 - (1-del)*gst*bet) , 0 , 0           , 0    , 0 , 0]; ...
         [0 , 0 , 0 , 0                             , 0 , 1           , 0    , 0 , 0]];

    GAM1 = [[(1-rho)*phi2, -(1-rho)*phi2 , 0       , 0 , 0 , 0        , 0 , 0 , rho]; ...
         [0   , 0 , 0       , 0 , 0 , qst/lst  , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , -qst/lst , 0 , 0 , 0]; ...
         [0   , 0 , kst/bet , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , alp     , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , 1        , 0 , 0 , 0];...
         [ (rho*mst/lst+rho^2-(kap1+kap2*rho+kap3*rho^2)) , 0 , -alp , 0 , 0 , -qst/lst-(kap1-kap2), -(kap2-kap3) , kap1 , 0]; ...         
         [ (rho^2-1)  , 1 , (1-alp)*(1-(1-del)*gst*bet)    , 0 , 0 , 0 , 0 , 0 , 0]; ...
         [ 0          , 0 , 0                              , 0 , 0 , 0 , 1 , 0 , 0]];

    C = zeros(nstate,1);
            	
	PPSI  = [[ -(1-rho)*phi2 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-kst/bet , 0 ]; ...         
          [-alp     , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]];
  
    PPI = [zeros(6,3);eye(3)];
    
elseif mspec == 5;
    %% Money Feedback Rule w/ output

    %% Define the vector of "state"-variables
    %% [dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    nstate = 9;

	%% Define Matrices of canonical system  
	%% 
	%% interest-hours rule:  ( 0 ~ -(1-rho)*phi2 ~ 0   ~ -(1-rho)*(1-alp)*phi1 ~ 0 ~   0     ~ 0 ~ 0 ~ 1)|
	%% interest-outout rule: ( 0 ~ -(1-rho)*phi2 ~ 0   ~ 0 ~ -(1-rho)*phi1 ~   0     ~ 0 ~ 0 ~ 1)|
	%% for money rules: phi = - phi

    phi1 = -phi1;
    phi2 = -phi2;
    
    GAM0 = [[1  , -(1-rho)*phi2 , 0   , 0 , -(1-rho)*phi1,   0     , 0 , 0 , 0]; ...            
           [mst/lst   , -1 , 0   , 0            , -1,   0     , 0 , 0 , 1]; ...
         [1-mst/lst , 0  , 0   , (1/(1-hst))  , 0 ,   0     , 0 , 0 , 0]; ...
         [cst       , -cst, kst , -(1-alp)*yst , 0 ,  0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , -(1-alp)     , 1 ,   0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , 0            , 0 ,   0     , 0 , 1 , 0]; ...
         [0 , 1 , 0 , (1-alp)                       , 0 , 0 , kap3 , 0 , 0]; ...
         [0 , 1 , 0 , (1-alp)*(1 - (1-del)*gst*bet) , 0 , 0           , 0    , 0 , 0]; ...
         [0 , 0 , 0 , 0                             , 0 , 1           , 0    , 0 , 0]];

    GAM1 = [[(1-rho)*phi2+rho, -(1-rho)*phi2 , 0       , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , qst/lst  , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , -qst/lst , 0 , 0 , 0]; ...
         [0   , 0 , kst/bet , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , alp     , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , 1        , 0 , 0 , 0];...
         [ (rho*mst/lst+rho^2-(kap1+kap2*rho+kap3*rho^2)) , 0 , -alp , 0 , 0 , -qst/lst-(kap1-kap2), -(kap2-kap3) , kap1 , 0]; ...         
         [ (rho^2-1)  , 1 , (1-alp)*(1-(1-del)*gst*bet)    , 0 , 0 , 0 , 0 , 0 , 0]; ...
         [ 0          , 0 , 0                              , 0 , 0 , 0 , 1 , 0 , 0]];

    C = zeros(nstate,1);
            	
	PPSI  = [[ -(1-rho)*phi2 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-kst/bet , 0 ]; ...         
          [-alp     , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]];
  
    PPI = [zeros(6,3);eye(3)];
    
elseif mspec == 6;
    %% Money Feedback Rule w/ hrs

    %% Define the vector of "state"-variables
    %% [dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    nstate = 9;

	%% Define Matrices of canonical system  
	%% 
	%% interest-hours rule:  ( 0 ~ -(1-rho)*phi2 ~ 0   ~ -(1-rho)*(1-alp)*phi1 ~ 0 ~   0     ~ 0 ~ 0 ~ 1)|
	%% interest-outout rule: ( 0 ~ -(1-rho)*phi2 ~ 0   ~ 0 ~ -(1-rho)*phi1 ~   0     ~ 0 ~ 0 ~ 1)|
	%% for money rules: phi = - phi

    phi1 = -phi1;
    phi2 = -phi2;
    
    GAM0 = [[1  , -(1-rho)*phi2 , 0   , -(1-alp)*(1-rho)*phi1, 0,  0     , 0 , 0 , 0]; ...            
           [mst/lst   , -1 , 0   , 0            , -1,   0     , 0 , 0 , 1]; ...
         [1-mst/lst , 0  , 0   , (1/(1-hst))  , 0 ,   0     , 0 , 0 , 0]; ...
         [cst       , -cst, kst , -(1-alp)*yst , 0 ,  0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , -(1-alp)     , 1 ,   0     , 0 , 0 , 0]; ...
         [0         , 0  , 0   , 0            , 0 ,   0     , 0 , 1 , 0]; ...
         [0 , 1 , 0 , (1-alp)                       , 0 , 0 , kap3 , 0 , 0]; ...
         [0 , 1 , 0 , (1-alp)*(1 - (1-del)*gst*bet) , 0 , 0           , 0    , 0 , 0]; ...
         [0 , 0 , 0 , 0                             , 0 , 1           , 0    , 0 , 0]];

    GAM1 = [[(1-rho)*phi2+rho, -(1-rho)*phi2 , 0       , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , qst/lst  , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , -qst/lst , 0 , 0 , 0]; ...
         [0   , 0 , kst/bet , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , alp     , 0 , 0 , 0        , 0 , 0 , 0]; ...
         [0   , 0 , 0       , 0 , 0 , 1        , 0 , 0 , 0];...
         [ (rho*mst/lst+rho^2-(kap1+kap2*rho+kap3*rho^2)) , 0 , -alp , 0 , 0 , -qst/lst-(kap1-kap2), -(kap2-kap3) , kap1 , 0]; ...         
         [ (rho^2-1)  , 1 , (1-alp)*(1-(1-del)*gst*bet)    , 0 , 0 , 0 , 0 , 0 , 0]; ...
         [ 0          , 0 , 0                              , 0 , 0 , 0 , 1 , 0 , 0]];

    C = zeros(nstate,1);
            	
	PPSI  = [[ -(1-rho)*phi2 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-kst/bet , 0 ]; ...         
          [-alp     , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]];
  
    PPI = [zeros(6,3);eye(3)];
    
elseif ( mspec == 101 ) | ( mspec == 102 )

    %% Clarida, gali, Gertler
	%% [y,inf,r2,r2star,g,z,xi_y,xi_infl]'
	%%     y  : output
	%%     inf: pi, inflation
	%%     r2 : R, nominal interest rate
	%%     r2star: R*, target interest rate
	%%     g  : government spending
	%%     z  : technology growth
	%%     xi_y: expected output
	%%     xi_infl: expected inflation
	%% define matrices of canonical system
    nstate = 8;

	neq  = nstate;
	nex  = 3;
	nend = 2;
	
	GAM0 = zeros(neq,neq);
	GAM1 = zeros(neq,neq);
	C = zeros(neq,1);
	PPSI = zeros(neq,nex);
	PPI = [zeros(neq-nend,nend);eye(nend)];
	
	%% IS Curve 
	GAM0(1,1)   = 1;
	GAM0(1,3)   = 1/tau;
	GAM0(1,5)   = -(1-rhog);
	GAM0(1,6)   = -rhoz/tau;
	GAM0(1,7)   = -1;
	GAM0(1,8)   = -1/tau;
	
	%% Phillips Curve      
	GAM0(2,1)   = -kap;
	GAM0(2,2)   = 1;
	GAM0(2,5)   = kap;
	GAM0(2,8)   = -bet;  %%%%%% CHANGE
	
	%% Interest Rate Smoothing 
	GAM0(3,3)   = 1;
	GAM0(3,4)   = -(1-rhor);  
	GAM1(3,3)   = rhor;
	PPSI(3,1)    = 1;
	
	%% Policy Rule 
	GAM0(4,1)   = -psi2;
	GAM0(4,4)   = 1;
	if (mspec == 101)
	    GAM0(4,2) = -psi1;
    elseif (mspec == 102)
    	GAM0(4,8) = -psi1;
	end
	
	%% Government Spending 
	GAM0(5,5)    = 1;
	GAM1(5,5)    = rhog;
	PPSI(5,2)     = 1;
	
	%% Technology Growth 
	GAM0(6,6)    = 1;
	GAM1(6,6)    = rhoz;
	PPSI(6,3)     = 1;
	
	%% Expected Output 
	GAM0(7,1)    = 1;
	GAM1(7,7)    = 1;
	
	%% Expected Inflation 
	GAM0(8,2)    = 1;
	GAM1(8,8)    = 1;
    
    
else   %% use standard cash-in-advance model 
	
	%% Define the vector of "state"-variables
	%% (dm(t), dp(t), dk(t+1), dh(t), dy(t), dc(t), dl(t), dd(t))
	
	
	GAM0 = [[1    , 0   , 0   , 0            , 0   , 0  , 0    , 0   ]; ...
          [1    , 0   , 0   , (1/(1-hst))  , 0   , 0  , -1   , 0   ]; ...
          [1    , -1  , 0   , 0            , 0  , -1  , 0    , 0   ]; ...
          [mst  , 0   , 0   , 0            , 0   , 0  , -lst , dst ]; ...
          [0    , 0   , 1 , -(1-alp)*(yst/kst) , 0 , (cst/kst)  , 0    , 0   ]; ...
          [0    , 0   , 0   , -(1-alp)     , 1   , 0  , 0    , 0   ]; ...
          [-rho , 1   , 0   , (1-alp)      , 0   , 0  , -1   , 0   ]; ...
          [0 , 1 , 0 , (1-alp)*(1-(1-del)*gst*bet) , 0 , 0 , 0  , 0]];
          
	
	GAM1 = [[rho , 0 , 0       , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , 0       , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , 0       , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , 0       , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , 1/bet   , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , alp     , 0 , 0 , 0 , 0 , 0    ]; ...
          [0   , 0 , -alp    , 0 , 0 , 0 , 0 , 0    ]; ...
          [ (rho^2-1) , 1 , (1-alp)*(1-(1-del)*gst*bet) , 0 , 0 , 0 , 0 , 0]];
	
	%% (dm(t), dp(t), dk(t+1), dh(t), dy(t), dc(t), dl(t), dd(t))
    
    C = zeros(8,1);
	
	
	PPSI  = [[ 0 , 1 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [ 0 , 0 ]; ...
          [-1/bet , 0 ]; ...         
          [-alp   , 0 ]; ...
          [ alp   , 0 ]; ...
          [ 0     , 0 ]];
  
    PPI = [zeros(7,1);eye(1)];
    
end

[T1,TC,T0,M,TZ,TY,gev,RC]=gensys(GAM0,GAM1,C,PPSI,PPI,1);

if (RC(1) ~= 1) | (RC(2) ~= 1);
   TTT = zeros(nstate,nstate);
   RRR = zeros(nstate,2);
   valid = - 3;
   return;
end;

TTT =    real( T1 );
RRR =    real( T0 );

 

if mspec <= 6
    %% Add additional variables dm(t-1) dp(t-1) dy(t-1)
	TTTins = zeros(3,3+size(TTT,2));
	TTTins(1,3+1) = 1;
	TTTins(2,3+2) = 1;
	TTTins(3,3+5) = 1;
	TTT = [TTTins ; ...
          [zeros(rows(TTT),3),TTT]];
	RRR = [zeros(3,size(RRR,2));RRR];

elseif ( mspec == 101 ) | ( mspec == 102 )
	%% Augment the state vector by y(t-1)
	
	TTT = [[TTT,zeros(size(TTT,1),1)]; ...
           [1,zeros(1,size(TTT,2))]];
	RRR = [RRR ; zeros(1,size(RRR,2))];

end   
