
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsgepriofed;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);

outfile00 = strcat(spath,'altable.tex');		% output file name 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
nobs =  nobsall;

fixparam = fixparamseq(1);
if fixparam ~= 0 
    fixparam
    break
end

%% set date
Idate = find(ti == dates);
tiall     = [ti(1:end-1);(ti(end):0.25:(ti(end)+.25*qahead))'];
datesall = strcat(num2str(floor(tiall)),'-',num2str(round(1+4*(tiall-floor(tiall)))));  



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      forecasts 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid00 = fopen(outfile00,'w');		% output file (set to write)

            
%% settings
percent = .68;
L = 0;

%% compute forecasts means and bands
Means = zeros(nvar,qahead,length(lambdaseq));
Bands = zeros(2,nvar,qahead,length(lambdaseq));

fprintf(fid00,'\n\n Baseline ');
for lambdas = lambdaseq     
    L = L+1;
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    
    %% open read path files 
    infile = strcat(spath,'/pred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode));
    num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
    numb = nburn*nvar*qahead*8;                  % number of bites to dicard


    fid = fopen(infile,'r');		% input file (set to read)               
    status = fseek(fid,numb,'bof');

    ypath = [];
	while ( ftell(fid) < num ) 
        % Read blocks of size nsim
		ypathadd = fread(fid,[nsim,nvar*qahead],'double');
        if all(size(ypathadd,2) == [nvar*qahead])
            ypath = [ypath;ypathadd];
        else
   			fclose(fid);
        end
        clear ypathdd;
	
	end;
	fclose(fid);
    ndraws = size(ypath,1);

    for I = 1:nvar
        yp = ypath(:,qahead*(I-1)+1:qahead*I);
        if cum_for(I)
           %% transform from log growth rates to % growth rates (annualized)
           yp = 100*4*(exp(yp/100)-1);
        end;
        
		Means(I,:,L) = mean(yp,1);
		Bands(:,I,:,L) = hpdint(yp,percent,1);

    end
    

end         % end lambdaseq



Alt = menu('Do you have an alternative forecast scenario?','Yes','No')
if Alt == 1
	%% compute forecasts means and bands for alternative forecast scenario 
	AMeans = zeros(nvar,qahead,length(lambdaseq));
	ABands = zeros(2,nvar,qahead,length(lambdaseq));
    
    L = 0;
	for lambdas = lambdaseq     
        L = L+1;
        
        fprintf(1,'\n Lambda = %2.3f',[lambdas]);
        
        %% open read path files 
        infile = strcat(spath,'Apred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode))
        num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
        numb = nburn*nvar*qahead*8;                  % number of bites to dicard
	
	
        fid = fopen(infile,'r');		% input file (set to read)               
        status = fseek(fid,numb,'bof');
	
        ypath = [];
		while ( ftell(fid) < num ) 
            % Read blocks of size nsim
			ypathadd = fread(fid,[nsim,nvar*qahead],'double');
            if all(size(ypathadd,2) == [nvar*qahead])
                ypath = [ypath;ypathadd];
            else
       			fclose(fid);
            end
            clear ypathdd;
		
		end;
		fclose(fid);
        ndraws = size(ypath,1);
	
        for I = 1:nvar
            yp = ypath(:,qahead*(I-1)+1:qahead*I);
            if cum_for(I)
               %% transform from log growth rates to % growth rates (annualized)
               yp = 100*4*(exp(yp/100)-1);
            end;
         
			AMeans(I,:,L) = mean(yp,1);
			ABands(:,I,:,L) = hpdint(yp,percent,1);
	
        end
	
	end         % end lambdaseq
    
end


%% do plots

previous = 4+round(1+4*(ti(Idate)-floor(ti(Idate))))-4;
future = 8+4-round(1+4*(ti(Idate)-floor(ti(Idate))))+4;

%previous = 16+round(1+4*(ti(Idate)-floor(ti(Idate))));
%future = 0
%pause 

sirf = (Idate-previous+1:1:Idate+future);
Mdates = datesall((Idate-previous+1:1:Idate+future),:);
intx = datesall(sirf,:);
intx = intx(1:4:end,:);

titlenames = varnames;


%% set YLim
ylims = zeros(nvar,2);
ylims(1,:) = [-2 7];        %% output
ylims(2,:) = [-1 5];        %% infl
ylims(3,:) = [0 7];        %% fed fund

ylims = []

Ys = YYall(1:Idate,:);
for I = 1:nvar
    if cum_for(I)
       %% transform from log growth rates to % growth rates (annualized)
       Ys(:,I) = 100*4*(exp(Ys(:,I)/100)-1);
    end;
end


L = 0;
MMeans = zeros(nvar,size(Mdates,1),length(lambdaseq));
for lambdas = lambdaseq     
    L = L+1;
    
    
    for V = 1:nvar
        figure(100);
        subplot(length(lambdaseq),nvar,V+nvar*(L-1));   
        mlp = [Ys(:,V);squeeze(Means(V,:,L))'];
        MMeans(V,:,L) = mlp(Idate-previous+1:1:Idate+future);

        bandsp = [repmat(Ys(:,V),[1 2]);squeeze(Bands(:,V,:,L))'];
        
        P = plot(sirf,mlp(sirf),'r-');
        set(P,'LineWidth',2.2);
        hold on;
        P = plot(sirf,bandsp(sirf,1),'r:',sirf,bandsp(sirf,2),'r:');
        set(P,'LineWidth',1);
        if Alt == 1
            
            Amlp = [Ys(:,V);squeeze(AMeans(V,:,L))'];
            Abandsp = [repmat(Ys(:,V),[1 2]);squeeze(ABands(:,V,:,L))'];
            
            P = plot(sirf,Amlp(sirf),'b-.');
            set(P,'LineWidth',1.5);
            hold on;
            P = plot(sirf,Abandsp(sirf,1),'b-.',sirf,Abandsp(sirf,2),'b-.');
            set(P,'LineWidth',.75);
            
        end

        %% make actual black
        mlp = [Ys(:,V);NaN*squeeze(Means(V,:,L))'];
        P = plot(sirf,mlp(sirf),'k-');
        set(P,'LineWidth',2.2);
        
        hold off;
        
        if L == 1
            title(titlenames(V));
        end
        if V == 1 
            ylabel(strcat('\lambda = ',num2str(lambdas)));
        end
        if ~isempty(ylims)
            set(gca,'YLim',ylims(V,:));
        end
        
        set(gca,'XLim',[min(sirf),max(sirf)]);
        set(gca,'XTick',sirf(1:4:end));
        set(gca,'XTickLabel',intx);
        grid on;
        
    end   

	fprintf(fid00,'\n\n lambda: %2.2f ',[lambdas]);
	fprintf(fid00,'\n dates ');
	fprintf(fid00,' %9.99s ',varnames{:});
    for ip = 1:size(Mdates,1)
        fprintf(fid00,'\n %9.99s',Mdates(ip,:));
    	fprintf(fid00,' %2.4f ',MMeans(:,ip,L));
    end

end         % end lambdaseq



figure(100)
titletext = input('\n title for forecast plots? ');
[ax,h]=subtitle(titletext);

titlefig = input('\n figure title ');
ng = strcat(gpath,lmodel,lprior,dataset,new,st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(Idate)),num2str(JustMode),titlefig,'for');
saveas(figure(100),ng,'fig');

set(figure(100),'PaperPosition',[0 0 8 6])
print(char(strcat(ng,'.eps')),'-deps');



CH = menu('Plot mean and st dev?','Yes','No');
close all;
if CH == 2
    fclose(fid00);		% close output file
    break
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      forecasts of standard deviations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% set grid
ngrid = 100;
sgridlim = [    [0 6];    ...
                [0 6];      ...
                [0 6]];
sgrid_all = [   (sgridlim(1,1):((sgridlim(1,2)-sgridlim(1,1))/(ngrid-1)):sgridlim(1,2))', ...
                (sgridlim(2,1):((sgridlim(2,2)-sgridlim(2,1))/(ngrid-1)):sgridlim(2,2))', ...
                (sgridlim(3,1):((sgridlim(3,2)-sgridlim(3,1))/(ngrid-1)):sgridlim(3,2))'];

mgridlim = [    [0 8];    ...
                [-1 8];      ...
                [0 9]];
mgrid_all = [   (mgridlim(1,1):((mgridlim(1,2)-mgridlim(1,1))/(ngrid-1)):mgridlim(1,2))', ...
                (mgridlim(2,1):((mgridlim(2,2)-mgridlim(2,1))/(ngrid-1)):mgridlim(2,2))', ...
                (mgridlim(3,1):((mgridlim(3,2)-mgridlim(3,1))/(ngrid-1)):mgridlim(3,2))'];

pdfsdtraj = zeros(ngrid,length(lambdaseq),nvar);
pdfmtraj = zeros(ngrid,length(lambdaseq),nvar);

outsdtraj = zeros(5,length(lambdaseq),nvar);
outmtraj = zeros(5,length(lambdaseq),nvar);

%% compute st dev distributions
for L = 1:length(lambdaseq)
    lambdas = lambdaseq(L)
    
    %% open read path files 
    infile = strcat(spath,'/pred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode));
    num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
    numb = nburn*nvar*qahead*8;                  % number of bites to dicard


    fid = fopen(infile,'r');		% input file (set to read)               
    status = fseek(fid,numb,'bof');

    ypath = [];
	while ( ftell(fid) < num ) 
        % Read blocks of size nsim
		ypathadd = fread(fid,[nsim,nvar*qahead],'double');
        if all(size(ypathadd,2) == [nvar*qahead])
            ypath = [ypath;ypathadd];
        else
   			fclose(fid);
        end
        clear ypathdd;
	
	end;
	fclose(fid);
    ndraws = size(ypath,1);

    for I = 1:nvar

        yp = [1+3*cum_for(I)]*ypath(:,qahead*(I-1)+1:qahead*I);

        sdtraj = std(yp(:,1:end),1,2);
        mtraj = mean(yp(:,1:end),2);
        
        sbwidth = 1.06*ndraws^(-1/5)*std(sdtraj);
        for sdind = 1:ngrid;
            pdfsdtraj(sdind,L,I) = (1/(ndraws*sbwidth))*sum(normpdf((sdtraj-sgrid_all(sdind,I))/sbwidth));
        end
        sdband = hpdint(sdtraj,.68,0);
        outsdtraj(:,L,I) = [median(sdtraj) mean(sdtraj) std(sdtraj) sdband'];
        
        mbwidth = 1.06*ndraws^(-1/5)*std(mtraj);
        for sdind = 1:ngrid;
            pdfmtraj(sdind,L,I) = (1/(ndraws*mbwidth))*sum(normpdf((mtraj-mgrid_all(sdind,I))/mbwidth));
        end
        mband = hpdint(mtraj,.68,0);
        outmtraj(:,L,I) = [median(mtraj) mean(mtraj) std(mtraj) mband'];

    end

   
end

Alt = menu('Do you have an alternative forecast scenario?','Yes','No')
if Alt == 1
	%% compute forecasts means and bands for alternative forecast scenario 
    
	Apdfsdtraj = zeros(ngrid,length(lambdaseq),nvar);
	Apdfmtraj = zeros(ngrid,length(lambdaseq),nvar);
    
    Aoutsdtraj = zeros(5,length(lambdaseq),nvar);
    Aoutmtraj = zeros(5,length(lambdaseq),nvar);

	
	%% compute st dev distributions
	for L = 1:length(lambdaseq)
        lambdas = lambdaseq(L)
        
        %% open read path files 
        infile = strcat(spath,'/Apred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode));
        num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
        numb = nburn*nvar*qahead*8;                  % number of bites to dicard
	
	
        fid = fopen(infile,'r');		% input file (set to read)               
        status = fseek(fid,numb,'bof');
	
        ypath = [];
		while ( ftell(fid) < num ) 
            % Read blocks of size nsim
			ypathadd = fread(fid,[nsim,nvar*qahead],'double');
            if all(size(ypathadd,2) == [nvar*qahead])
                ypath = [ypath;ypathadd];
            else
       			fclose(fid);
            end
            clear ypathdd;
		
		end;
		fclose(fid);
        ndraws = size(ypath,1);
	
        for I = 1:nvar
	
            yp = [1+3*cum_for(I)]*ypath(:,qahead*(I-1)+1:qahead*I);
	
            sdtraj = std(yp(:,1:end),1,2);
            mtraj = mean(yp(:,1:end),2);
            
            sbwidth = 1.06*ndraws^(-1/5)*std(sdtraj);
            for sdind = 1:ngrid;
                Apdfsdtraj(sdind,L,I) = (1/(ndraws*sbwidth))*sum(normpdf((sdtraj-sgrid_all(sdind,I))/sbwidth));
            end
            sdband = hpdint(sdtraj,.68,0);
            Aoutsdtraj(:,L,I) = [median(sdtraj) mean(sdtraj) std(sdtraj) sdband'];
                                                                  
            mbwidth = 1.06*ndraws^(-1/5)*std(mtraj);
            for sdind = 1:ngrid;
                Apdfmtraj(sdind,L,I) = (1/(ndraws*mbwidth))*sum(normpdf((mtraj-mgrid_all(sdind,I))/mbwidth));
            end
            mband = hpdint(mtraj,.68,0);
            Aoutmtraj(:,L,I) = [median(mtraj) mean(mtraj) std(mtraj) mband'];
            
        end
	
       
	end
    
end
%% plots: st dev


ylims = zeros(2,length(lambdaseq),nvar);
for L = 1:length(lambdaseq)
    for I=1:nvar
        ylims(2,L,I) = 1.01*max([squeeze(pdfsdtraj(:,L,I))]);    
        if Alt == 1
            ylims(2,L,I) = 1.01*max([squeeze(pdfsdtraj(:,L,I));squeeze(Apdfsdtraj(:,L,I))]);    
        end
    end
end

for L = 1:length(lambdaseq)

    lambdas = lambdaseq(L);
    for I = 1:nvar
        figure(101);
        subplot(length(lambdaseq),nvar,nvar*(L-1)+I);
        P = plot(sgrid_all(:,I),squeeze(pdfsdtraj(:,L,I)),'r');
        set(P,'LineStyle','-');
        set(P,'LineWidth',1.5);
        
        fprintf(fid00,'\n\n lambda: %2.2f ',[lambdas]);
        fprintf(fid00,'\n lambda: %9.99s ',varnames{I});
        fprintf(fid00,'\n median mean  stdev 68low  68high \n');
        fprintf(fid00,' %2.4f ',outsdtraj(:,L,I));
        
        if Alt == 1
            hold on;
            P = plot(sgrid_all(:,I),squeeze(Apdfsdtraj(:,L,I)),'b');
            set(P,'LineStyle',':');
            set(P,'LineWidth',1.5);
            
            fprintf(fid00,'\n\n Alternative Policy ');
            fprintf(fid00,'\n median mean  stdev 68low  68high \n');
            fprintf(fid00,' %2.4f ',Aoutsdtraj(:,L,I));
        
        end
        hold off;
            

        set(gca,'XLim',sgridlim(I,:));
        set(gca,'YLim',squeeze(ylims(:,L,I)));

        
        if L == 1
            title(varnames{I});
        end
        if I == 1
            if isinf(lambdas)
                ylabel('\lambda = \infty');
            else
                ylabel(strcat('\lambda = ',num2str(lambdas)));
            end
        end
        
        hold off;
        
        
    end
   
end
figure(101)
[ax,h]=subtitle(titletext);

ng=strcat(gpath,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),new,st,titlefig,'polstd');

saveas(figure(101),ng,'fig');

set(figure(101),'PaperPosition',[0 0 8 6])
print(char(strcat(ng,'.eps')),'-deps');


%% plots: mean

ylims = zeros(2,length(lambdaseq),nvar);
for L = 1:length(lambdaseq)
    for I=1:nvar
        ylims(2,L,I) = 1.01*max([squeeze(pdfmtraj(:,L,I))]);    
        if Alt == 1
            ylims(2,L,I) = 1.01*max([squeeze(pdfmtraj(:,L,I));squeeze(Apdfmtraj(:,L,I))]);    
        end
    end
end

for L = 1:length(lambdaseq)

    lambdas = lambdaseq(L);
    for I = 1:nvar
        figure(102);
        subplot(length(lambdaseq),nvar,nvar*(L-1)+I);
        P = plot(sgrid_all(:,I),squeeze(pdfmtraj(:,L,I)),'r');
        set(P,'LineStyle','-');
        set(P,'LineWidth',1.5);

        if Alt == 1
            hold on;
            P = plot(sgrid_all(:,I),squeeze(Apdfmtraj(:,L,I)),'b');
            set(P,'LineStyle',':');
            set(P,'LineWidth',1.5);
        end
        hold off;
        
        set(gca,'XLim',sgridlim(I,:));
        set(gca,'YLim',squeeze(ylims(:,L,I)));

        
        if L == 1
            title(varnames{I});
        end
        if I == 1
            if isinf(lambdas)
                ylabel('\lambda = \infty');
            else
                ylabel(strcat('\lambda = ',num2str(lambdas)));
            end
        end
        
        hold off;
        
        
    end
   
end

figure(102)
[ax,h]=subtitle(titletext);

set(figure(102),'PaperPosition',[0 0 8 6])
ng=strcat(gpath,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),new,st,titlefig,'polm.eps');
print(char(ng),'-deps');



fclose(fid00);		% close output file

pause;

close all;
