function [p,fret] = frprmn(p,ftol,func,dfunc,varargin)
% given starting point p, a function func that needs to be minimized, and the function dfunc that computes the 
% gradient of func at any arbitrary point, frprmn performs Fletcher-Reeves/Polak-Ribiere
% ftol is the convergence tolerance. returns p (minimum, if attained) and corresponding value of func (fret)
% taken from fortran routine with the same name (see Numerical Recipes by Press et al...)

% parameters
NMAX = 50;
ITMAX = 200; % max number of iterations, change if needed
EPS = 1e-10;
Verbose = 1;

fp = eval([func '(p,varargin{:})']);
[xi badg] = eval([dfunc '(p,varargin{:})']);

% Initialize
g = -xi;
h = g;
xi = h;

for its = 1:ITMAX
    iter = its;
    [p,fret] = linmin(p,xi,func,varargin{:});
    if 2*abs(fret-fp) <= ftol*(abs(fret)+abs(fp)+EPS) % test if done        
        if Verbose; disp(sprintf('FRPR done, iteration: %2.2f',iter)); end;
        if iter>1; return; end;
    end
    fp = fret;
    [xi badg] = eval([dfunc '(p,varargin{:})']);

    gg = sum(g.^2);
    %dgg = sum(xi.^2); % for Fletcher-Reeves
    dgg = (xi+g)'*xi;  % for Polak-Ribiere
    if gg == zeros(size(gg)) % if determinant is zero, you're done (unlikely)
        return
    end
    gam = dgg/gg;
    g  = -xi;
    h = g+gam*h;
    xi = h;
end
disp('brent exceeded maximum number of iterations');
pause

        
    