close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%run startdsgepriofed;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
nobs =  nobsall;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% if JustMode: do not do simulations and compute forecasts at mode
DOPLOTSONLY = 1;

nirf = 41;
%% cumulative irf
ccum_irf = [];
for II=1:length(cum_irf)
    ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end

%% set date
I = find(ti == dates);

%%  Configure the Metropolis Algorithm
if JustMode
    nblocks = 1;
    nsim = 100;
end


Verbose = 0;

%if ~DOPLOTSONLY
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% do Lambda loop
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	for fixparam = fixparamseq             
        for lambdas = lambdaseq     
            
                fprintf(1,'\n Lambda = %2.3f',[lambdas]);
        		fprintf(1,'\n Time %3.2f ',[ti(I)]);
	
                %% open write files
                %% params file
                infile1 = strcat(spath,'mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
                fid1 = fopen(infile1,'r');		% output file (set to read)   
                num1 = nblocks*nsim*npara*8;            % number of bites per file
                
                %% VAR file
                infile2 = strcat(spath,'varpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
                fid2 = fopen(infile2,'r');		% output file (set to read)   
                num2 = nblocks*nsim*(nvar*(1+nlags*nvar)+nvar^2)*8;            % number of bites per file
                
                %% Irf VAR file
                outfile4 = strcat(spath,'/irf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
                fid4 = fopen(outfile4,'w');		% output file (set to write)   
                %% Irf model file
                outfile5 = strcat(spath,'/virf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
                fid5 = fopen(outfile5,'w');		% output file (set to write)   
                
                %% set initial values for simulations
                
                u= 0;
				%for iblock = 1:nblocks  
                
                while ( ftell(fid1) < num1 )             
                    varpara = fread(fid2,[nsim,nvar*(1+nlags*nvar)+nvar^2],'double');        
                    parasimm = fread(fid1,[nsim,npara],'double');

                    irfs = zeros(size(varpara,1),nvar^2*nirf);
                    virfs = zeros(size(varpara,1),nvar^2*nirf);
                        
                    for j = 1:size(varpara,1)
                        params = parasimm(j,:)';
                        
                        cct_sim = reshape(varpara(j,1:nvar*(1+nlags*nvar)),1+nlags*nvar,nvar)'; 
                        sig_sim = reshape(varpara(j,nvar*(1+nlags*nvar)+1:end),nvar,nvar);  
                    
                        %% model's irf
                       	[TTT, RRR, valid] = dsgesolv(params,mspec);
                        aairf = irfdsge(TTT,RRR,params,nvar,mspec,npara,nirf,cum_irf);
                        
                   		a0_m = [];
						for ii=1:nvar
							a0_m = [a0_m,aairf(1,nvar*(ii-1)+1:nvar*ii)'];
						end; 
                        
                        %% var's irf
                        	% Compute MA representation from companion form VAR
                        sig_chol = chol(sig_sim)';
						cc_tilded = [cct_sim(:,1+1:1+nlags*nvar) ; ...
                                    [eye(nvar*(nlags-1)) , zeros(nvar*(nlags-1),nvar)]];
					
						ccpower = eye(nvar*nlags);
						msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];
					
						bbirf    = zeros(nirf,nvar^2);
                        out = msel'*ccpower*msel*sig_chol;
                        out = out(:)';
						bbirf(1,:) = out;
					
					
						for t = 2:nirf;
					
							ccpower = cc_tilded*ccpower;
                            out = msel'*ccpower*msel*sig_chol;
                            out = out(:)';
							bbirf(t,:)  = out;
					
                        end;
					
                      	bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);
	
                        
                        %% this phimat rotates the VAR approximation to the MA so that the IRF on impact are the same
                        %% need lower triangular cholesky
                        [phimat,r_a] = qr(a0_m');
                        phimat = phimat';
                        irfs(j,:) = 100*aairf(:)';                      
                        bbirf = bbirf*kron(phimat,eye(nvar));       
                        virfs(j,:) = 100*bbirf(:)';       
                            
	
                    end;
                    
                    fwrite(fid4,irfs,'double');
                    fwrite(fid5,virfs,'double');
				end;		% end simulation - given lambda and time period
                fclose(fid1);
                fclose(fid2);
                num4 = ftell(fid4)
                fclose(fid4);
                num5 = ftell(fid5)
                fclose(fid5);
                
                %% save nums
                save(strcat(outfile4,'.mat'),'num4');	
                save(strcat(outfile5,'.mat'),'num5');	
        end         % end lambdaseq
	end % end fixparam
    
    %end
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PLOTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fixparam = 0

%% settings
csize = nirf*nvar^2;
nburn = ceil(nsim*nblocks/10);
percent = .90;
docov = 0;

tirf = (1:1:nirf)';
sirf = (1:17);

titlenames = varnames;
titlenames(cum_irf) = strcat(titlenames(cum_irf),' - cumulative');

intx = tirf(sirf);
    
Iirf = menu('Choose shock',names_shocks);
selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
L = 0;
%% normalization: normalize mon. pol. irf so that the impact on the Fed Fund is 25 basis points
norma = cell(3,1);
norma(1) = {1:nvar*nirf};      %% which irf (mon pol)
norma(2) = {25};                                        %% what value
norma(3) = {2*nirf+1};                                     %% what element (Fed fund on impact)

norma = []
 
%% set YLim
ylims = zeros(nvar,2);
ylims(1,:) = [-40 60];        %% output
ylims(2,:) = [-40 10];        %% infl
ylims(3,:) = [-20 30];        %% fed fund

ylims = [];

Legend102 = cell(length(lambdaseq),1);
Color102 = eye(max(length(lambdaseq),3)); Color102 = Color102(:,1:3);

for lambdas = lambdaseq     
    L = L+1;
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
	fprintf(1,'\n Time %3.2f ',[ti(I)]);

    %% open write files
    %% Irf VAR file
    infile4 = strcat(spath,'/irf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid4 = fopen(infile4,'r');		% output file (set to write)   
    %% Irf model file
    infile5 = strcat(spath,'/virf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid5 = fopen(infile5,'r');		% output file (set to write)   
    
    load(strcat(infile4,'.mat'),'num4');	
    load(strcat(infile5,'.mat'),'num5');
    
    %%%% compute means and bands - Model
    [m,s,c,ndraws,bands] = pmom(fid4,csize,nsim,nblocks,nburn,percent,docov,norma);
    size(m)
    ndraws
    %% select irf
    msel = m(selirf);
    bandssel = bands(selirf,:);
    
    %%%% compute means and bands - VAR
    [mv,sv,cv,ndrawsv,bandsv] = pmom(fid5,csize,nsim,nblocks,nburn,percent,docov,norma);
    %% select irf
    mselv = mv(selirf);
    bandsselv = bandsv(selirf,:);

    for V = 1:nvar
        figure(101);
        subplot(length(lambdaseq),nvar,V+nvar*(L-1));   
        mlp = msel(nirf*(V-1)+1:nirf*V);
        bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
        P = plot(intx,mlp(sirf),'-');
        set(P,'LineWidth',1);
        hold on;
        %P = plot(intx,bandsp(sirf,1),':',intx,bandsp(sirf,2),':');
        %set(P,'LineWidth',.3);
        %hold on;
        
        mlpv = mselv(nirf*(V-1)+1:nirf*V);
        bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
        P = plot(intx,mlpv(sirf),'-.');
        set(P,'LineWidth',2.2);
        hold on;
        P = plot(intx,bandspv(sirf,1),':',intx,bandspv(sirf,2),':');
        set(P,'LineWidth',1);
        
        hold off;
        
        if L == 1
            title(titlenames(V));
        end
        if V == 1 
            ylabel(strcat('\lambda = ',num2str(lambdas)));
        end
        if ~isempty(ylims)
            set(gca,'YLim',ylims(V,:));
        end
        
        set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
        set(gca,'XTick',intx([1 5:4:end]));
        set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));


        figure(102);
        subplot(1,nvar,V);   
        mlp = msel(nirf*(V-1)+1:nirf*V);
        bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
        %P = plot(intx,mlp(sirf),'-');        set(P,'LineWidth',1);
        hold on;
        
        mlpv = mselv(nirf*(V-1)+1:nirf*V);
        bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
        P = plot(intx,mlpv(sirf),'-');
        set(P,'LineWidth',2.2);
        set(P,'Color',Color102(L,:));
        hold on;
        %P = plot(intx,bandspv(sirf,1),':',intx,bandspv(sirf,2),':'); set(P,'LineWidth',1);
        
        hold off;
        
        if L == 1
            title(titlenames(V));
        end
        if ~isempty(ylims)
            set(gca,'YLim',ylims(V,:));
        end
        Legend102{L} = strcat('\lambda = ',num2str(lambdas));
        
        set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
        set(gca,'XTick',intx([1 5:4:end]));
        set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));
        
    end   



end         % end lambdaseq

figure(102);
legend(Legend102);
hold off;
    
set(figure(101),'PaperPosition',[0 0 8 6])
ng=strcat(gpath,lmodel,lprior,dataset,new,st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(I)),num2str(JustMode),names_shocks(Iirf),'irf');


%print(figure(101));
%print(figure(102));

%saveas(figure(101),char(ng),'fig');
%print(char(strcat(ng,'.eps')),'-deps');


pause
close all;

for lambdas = lambdaseq     
   L = L+1;
   
   fprintf(1,'\n Lambda = %2.3f',[lambdas]);
   fprintf(1,'\n Time %3.2f ',[ti(I)]);
   
   for Iirf = 1:nvar
   
	   selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
	   L = 0;
	   %% normalization: normalize mon. pol. irf so that the impact on the Fed Fund is 25 basis points
	   norma = cell(3,1);
	   norma(1) = {1:nvar*nirf};      %% which irf (mon pol)
	   norma(2) = {25};                                        %% what value
	   norma(3) = {2*nirf+1};                                     %% what element (Fed fund on impact)
	   
	   norma = []
	   
	   %% set YLim
	   ylims = zeros(nvar,2);
	   ylims(1,:) = [-40 60];        %% output
	   ylims(2,:) = [-40 10];        %% infl
	   ylims(3,:) = [-20 30];        %% fed fund
	   
	   ylims = [];
	   
	   Legend102 = cell(length(lambdaseq),1);
	   Color102 = eye(max(length(lambdaseq),3)); Color102 = Color102(:,1:3);
	   
	   
	   
	   %% open write files
	   %% Irf VAR file
	   infile4 = strcat(spath,'/irf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
	   fid4 = fopen(infile4,'r');		% output file (set to write)   
	   %% Irf model file
	   infile5 = strcat(spath,'/virf',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
	   fid5 = fopen(infile5,'r');		% output file (set to write)   
	   
	   load(strcat(infile4,'.mat'),'num4');	
	   load(strcat(infile5,'.mat'),'num5');
	   
	   %%%% compute means and bands - Model
	   [m,s,c,ndraws,bands] = pmom(fid4,csize,nsim,nblocks,nburn,percent,docov,norma);
	   size(m)
	   ndraws
	   %% select irf
	   msel = m(selirf);
	   bandssel = bands(selirf,:);
	   
	   %%%% compute means and bands - VAR
	   [mv,sv,cv,ndrawsv,bandsv] = pmom(fid5,csize,nsim,nblocks,nburn,percent,docov,norma);
	   %% select irf
	   mselv = mv(selirf);
	   bandsselv = bandsv(selirf,:);
	   
	   for V = 1:nvar
	   	   figure(1001);
	   	   subplot(nvar,nvar,V+nvar*(Iirf-1));   
	   	   mlp = msel(nirf*(V-1)+1:nirf*V);
	   	   bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
	   	   P = plot(intx,mlp(sirf),'-');
	   	   set(P,'LineWidth',1);
	   	   hold on;
	   	   %P = plot(intx,bandsp(sirf,1),':',intx,bandsp(sirf,2),':');
	   	   %set(P,'LineWidth',.3);
	   	   %hold on;
	   	   
	   	   mlpv = mselv(nirf*(V-1)+1:nirf*V);
	   	   bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
	   	   P = plot(intx,mlpv(sirf),'-.');
	   	   set(P,'LineWidth',2.2);
	   	   hold on;
	   	   P = plot(intx,bandspv(sirf,1),':',intx,bandspv(sirf,2),':');
	   	   set(P,'LineWidth',1);
	   	   
	   	   hold off;
	   	   
	   	   if Iirf == 1
		   	   title(titlenames(V));
	   	   end
	   	   if V == 1 
		   	   ylabel(names_shocks(Iirf));
	   	   end
	   	   if ~isempty(ylims)
	   		   set(gca,'YLim',ylims(V,:));
	   	   end
	   	   
	   	   set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
	   	   set(gca,'XTick',intx([1 5:4:end]));
	   	   set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));
	   	   
	   
	   
	   end   
	   
	   
	   
   end; %% end shock
   
   set(figure(1001),'PaperPosition',[0 0 8 6])
   ng=strcat(gpath,lmodel,lprior,dataset,new,st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(I)),num2str(JustMode),num2str(100*lambdas),'irfallshocks');
 %%saveas(figure(1001), three, 'fig');
   
   %%print(figure(1001));
%%save figure(1001);


%%saveas(figure(1001),ng,'fig');
 %  print(char(strcat(ng,'.eps')),'-deps');
   
  % pause
   
end         % end lambdaseq

%clear all;
%close all;
