function [p,fret] = linmin(p,xi,func,varargin);
% linmin: line minimization along a direction p+xi
% inputs: p = nx1 initial point; xi = nx1 direction; func = routine computing the function that is to be maximized
% varargin = inputs for func (other than point at which is evaluated)
% outputs: p = argmax , fret = value of function at max
% taken from fortran routine with the same name (see Numerical Recipes by Press et al...)

% Parameters
NMAX = 50; % expected # of iterations
TOL = 1e-4; % tolerance (input for Brent)

% Initial guesses for brackets
ax = 0;
bx = 1;

% Bracket the minimum
[ax,bx,cx,fa,fb,fc] = mnbrak(ax,bx,p,xi,func,varargin{:});
% Proceed with Brent 
[xmin,fret] = brent(ax,bx,cx,TOL,p,xi,func,varargin{:});
% Compute new p
p = p+xmin*xi;