%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end


% Time series, extracted sometimes beginning 2002 (have final GDP 2001:III)
% Gross Domestic Product (SAAR, Bil.Chn.1996$), Bureau of Economic Analysis
% CPI-U: All Items (SA, 1982-84=100), Bureau of Labor Statistics
% Quarterly Data from 1947:I to  (depending on data set)
% Fed Fund rate, quarterly from 1954:III to 2001:III
start_date = 1947.00;
%start_date = 1957.00;

fname = strcat(fpath,'data',num2str(floor(dataset)),'.txt')
series = importdata(fname);
series(:,1:2) = log(series(:,1:2));
%%series(:,3) = log(series(:,3)); ffr in logs


nobs   = size(series,1);
ti     = (start_date:0.25:(start_date+.25*(nobs-1)))';

%% Compute growth rates
series(2:end,1:2) = 100*(series(2:end,1:2) - series(1:end-1,1:2));
%series(3:end,2)= series(3:end,2)-series(2:end-1,2); %diff 2
%series(2:end,3)=series(2:end,3)*(1/100); same scale for ffr

series = series(2:end,:);

if  round(10*(dataset-floor(dataset))) == 1  
    series = [100*series(:,1:2),series(:,3)];
elseif round(10*(dataset-floor(dataset))) == 2
  % series = [100*series(:,1:2),series(:,4)];    
elseif round(10*(dataset-floor(dataset))) == 0
    series = [series(:,1:2)];    
end


nobs   = nobs-1;
ti     = ti(2:end);

%% take into account that series for Fed Fund is shorter
if any(isnan(series(:,end)));
    Init = max(find(isnan(series(:,end))))+1;
else
    Init = 1;
end;
Init=26;
series = series(Init:end,:);
ti     = ti(Init:end);
nobs = size(series,1);

YYall     = series(1+T0:nobs,:);            % Real GDP Growth | CPI QoQ | FFR 
XXall     = ones(nobs-T0, 1+nvar*nlags);    % Constant + Lags (see below)
ti     = ti(1+T0:nobs,:);

for i = 1:1:nlags;
   XXall(:,1+(i-1)*nvar+1:1+i*nvar) = series(T0-i+1:nobs-i,:);
end

Tend = nobs-Tend;
k = 1+nvar*nlags;

YY     = YYall(1:Tend-T0,:);
XX     = XXall(1:Tend-T0,:);
if Tend-T0 == size(XXall,1)
    %% if you're predicitng from the end of the sample (Tend set = 0)
    YYfut  = [];
    XXpred = [1,YYall(end,:),XXall(end,1+1:k-nvar)];      
else
    YYfut  = YYall(Tend-T0+1:size(YYall,1),:);
    XXpred = XXall(Tend-T0+1,:);
end;

YYpre = YYall(Tend-T0-7:Tend-T0,:);

YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

nobs = size(YY,1);
nobsall = size(YYall,1)

%% set nobsfut
nobsfut = 16;

yyfut = NaN*ones(nobsfut,size(YYfut,2));
yyfut(1:size(YYfut,1),:) = YYfut;
YYfut = yyfut;

dataset = num2str(10*dataset);

datt = [ti,YYall];
FORMAT = '\n %2.2f  '; 
for i = 1:nvar
    FORMAT = strcat(FORMAT,' %2.6f ');
end
fprintf(1,'\n\n Actual data  \n\n');
fprintf(1,FORMAT,datt');


ti(end-stime)
ti(end)

fname = strcat(spath,'dataout',dataset);
save(fname,'YYYY')
save(fname,'XXYY','-append')
save(fname,'XXXX','-append')
save(fname,'nobs','-append')
save(fname,'YY','-append')
save(fname,'XX','-append')
save(fname,'ti','-append')
save(fname,'YYfut','-append')
save(fname,'XXpred','-append')
save(fname,'nobsfut','-append')
save(fname,'YYall','-append')
save(fname,'XXall','-append')
save(fname,'nobsall','-append')
save(fname,'YYpre','-append')


