%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc; close all; fclose('all'); pause(.001)
    randn('state', 180277)
    rand ('state', 180277)

loaddata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
% [fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

% Write formatted data to file
fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

% Find indices of nonzero elements
ICUM = find(cum_for);
INCUM = find(cum_for == 0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load information for prior distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');  %e.g. m1012pri
% Define Prior parameters
prior = feval(priorfile);

pmean  = prior(1:npara,1);  % get prior mean
pstdd  = prior(1:npara,2);  % get prior standard dev.
pshape = prior(1:npara,3);  % get prior distribution (from 1 to 4)

nonpolipar = [1:1:npara];   % non policy parameters
nonpolipar(polipar) = [];   

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
nobs =  nobsall;
XXall = [XXall;[1,YYall(end,:),XXall(end,1+1:1+nvar*nlags-nvar)]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% set date
I = find(ti == dates);

%%  Configure the Metropolis Algorithm
cc0     = 0.2;
cc      = 0.2;

Verbose = 0;

date_q = (1:1:qahead)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for fixparam = fixparamseq
    L = 0;
    for lambdas = lambdaseq
        L = L+1;
        lmodel
        lambdas
        oparaest2         = strcat(spath,lmodel,lprior,'para',new,st,num2str(100*lambdas),num2str(100*ti(I)));
        
        if ~exist(strcat(oparaest2,'.mat'),'file')
            params = pmean;
            save(oparaest2,'params');
        end

        fprintf(1,'\n\nRMSE computation: lambda = %2.3f',[lambdas]);
		fprintf(1,' %8.98s;',fixparam);
       		
        if stime                    
			YY     = YYall(I-stime+1:I,:);
			XX     = XXall(I-stime+1:I,:);             
        else
			YY     = YYall(1:I,:);
            XX     = XXall(1:I,:);                   
        end

        if I+qahead <= nobsall
			YYfut  = YYall(I+1:I+qahead,:);
            XXpred = XXall(I+1,:);
        elseif I == nobsall
			YYfut  = NaN*zeros(qahead,nvar);
            XXpred = XXall(end,:);
        elseif (I < nobsall) & (I+qahead > nobsall)
            Ii = nobsall-I
			YYfut  = [YYall(I+1:I+Ii,:);NaN*zeros(qahead-Ii,nvar)];
            XXpred = XXall(end-Ii,:);        
        end
		
        nobs = size(YY,1);

        lambda = lambdas*nobs;
        
		YYYY = YY'*YY;
		XXYY = XX'*YY;
		XXXX = XX'*XX;
%        'MHVARSIM - 116', keyboard

        if isinf(lambdas)
            inf = 'inf';
        else
            inf = '';
        end
            
        if (lambdas > 0) | isinf(lambdas)
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% find posterior mode wrt dsge parameters  
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


            if ~fixparam
                
                clear params*;
                
            	params = load(oparaest2,strcat('params',num2str(100*lambdas),num2str(100*ti(I))));
                if ( isfield(params,strcat('params',num2str(100*lambdas),num2str(100*ti(I)))) ) % & ( 100*ti(I) ~= 198675)
                    params = getfield(params,strcat('params',num2str(100*lambdas),num2str(100*ti(I))));                         

                    save(oparaest2,'params','-append');

                else
                    load(oparaest2,'params');          
                    
            		%%     Maximize the posterior density  
            		
					x0= invtrans(params,trspec)+0.01*randn(npara,1);
                    
                	H0= eye(npara)*1E-4;
					nit = 1000;
					crit= 1E-10;
                    [fh,xh,g,H,itct,fcount,retcode] = csminwel(strcat('objfcn',new,inf),x0,H0,[],crit,nit,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape);
                     
                    
                    params = trans(xh,trspec);     
                    if itct <4
                        % pause
                    end
                end
                
        
                save(oparaest2,'params','-append');
                assignin('base',strcat('params',num2str(100*lambdas),num2str(100*ti(I))),params);
                save(oparaest2,strcat('params',num2str(100*lambdas),num2str(100*ti(I))),'-append');
                
                
            elseif fixparam
             
				    params = pmean; 
            end;
            
       %params=params*0.01;
       
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% find Hessian corresponding to posterior mode  
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            obj = objfcn2( params,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,strcat('marglh',new) );
            
           	hessian = load(oparaest2,strcat('hessian',num2str(100*lambdas),num2str(100*ti(I))));
            if ( isfield(hessian,strcat('hessian',num2str(100*lambdas),num2str(100*ti(I)))) ) % & ( 100*ti(I) ~= 198675)
                hessian = getfield(hessian,strcat('hessian',num2str(100*lambdas),num2str(100*ti(I))));                         
            else
                hessian = hessi(strcat('objfcn2',inf),params,1,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,strcat('marglh',new));
            end
            assignin('base',strcat('hessian',num2str(100*lambdas),num2str(100*ti(I))),hessian);
            save(oparaest2,strcat('hessian',num2str(100*lambdas),num2str(100*ti(I))),'-append');
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% set up Metropolis Hastings
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            rankhhm = npara;
			% Create Inverse by Singular Value Decomposition
			[u,s,v] = svd(hessian);
			md = min(size(s));
			bigev = find(diag(s(1:md,1:md))>1e-6);
			u = u(:,bigev);
			v = v(:,bigev);
			s = s(bigev,bigev);
			
			invhhmdet = 1;
			
			for i = 1:npara;
				if i > rankhhm;
                    s(i,i) = 0;
				else;
                    s(i,i)    = 1/s(i,i);
                    invhhmdet = invhhmdet*s(i,i);
				end
			end
			
			invhhm  = u*s*u';
			sigmult = u*sqrt(s);
					             
            sigscale    =  sigmult;     clear sigmult;
            sigpropinv  =  hessian;     clear hessian*;
	
            % Hessian is of reduced rank. Keep zero rows and columns and do SVD
			
			%if mspec == 2;  %%% ask frank about this
			if mspec > 1;  %%% ask frank about this
               sigpropdim = npara;
			end
	
            [u,s,v] = svd(sigpropinv);
			md = min(size(s));
			bigev = find(diag(s(1:md,1:md))>1e-6);
			u = u(:,bigev);
			v = v(:,bigev);
			s = s(bigev,bigev);
			
			sigproplndet = 0;
			
			for i = 1:npara;
				if i <= sigpropdim;
					s(i,i)    = 1/s(i,i);
					sigproplndet = sigproplndet + log(s(i,i));
				end
			end
	
			statname = {'POST';'LIKE';'PROP';'PROPPOST';'REJECT'};
			
			
           	[TTT,RRR,valid] = dsgesolv(params,mspec); %  'mhvarsim - 240', keyboard
            [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara);
			[lnpost0,lnpy0] = feval(strcat('objfcnmh9',inf),params,bounds,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode);
			% Initialize PARANEW			
			valid = 0;
			while ~valid
				paranew   = params + cc0*(sigscale*randn(npara,1));
               	[TTT,RRR,valid] = dsgesolv(paranew,mspec);
                [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,paranew,nvar,nlags,mspec,npara);
				[postnew, likenew] = feval(strcat('objfcnmh9',inf),paranew,bounds,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode);
				
				propdens  = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc0^2) ...
                           -0.5*(paranew - params)'*sigpropinv*(paranew - params)/cc0^2;
				if postnew > -10000;
                  valid = 1;
				end
			end  
        end
        
        return
        if ~JustMode
tic
            fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    		fprintf(1,'\n Time %3.2f ',[ti(I)]);

            %% open write files
            %% params file
            outfile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid1 = fopen(outfile1,'w');		% output file (set to write)   
            %% VAR file
            outfile2 = strcat(spath,'/varpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid2 = fopen(outfile2,'w');		% output file (set to write)   
            %% Forecasts file
            outfile3 = strcat(spath,'/pred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid3 = fopen(outfile3,'w');		% output file (set to write)   
             %% Posterior file
            outfile4 = strcat(spath,'/post',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid4 = fopen(outfile4,'w');		% output file (set to write)   
            %% Rotation file
            outfile5 = strcat(spath,'/rot',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid5 = fopen(outfile5,'w');		% output file (set to write)   
           
            %% set initial values for simulations 
            
			for iblock = 1:nblocks  
				parasimm = zeros(nsim,npara);
				likesim = zeros(nsim,1);
				postsim = zeros(nsim,1);
				rej     = zeros(nsim,1);
				propsim = zeros(nsim,1);
				proppostsim = zeros(nsim,1);
                                   
				varpara = zeros(nsim,nvar*(1+nlags*nvar)+nvar^2);
    			pred    = zeros(nsim,nvar*qahead);
				rotpara = zeros(nsim,nvar^2);
                    
                for j = 1:nsim
                    
                    if  lambdas > 0 & ~fixparam
                        
                        if ( j == 1 ) & (iblock == 1)
     						postold      = postnew;
							likeold      = likenew;
							paraold      = paranew;     
                            rej(1) = 0;
                            yyyydold = yyyyd; 
                            xxyydold = xxyyd; 
                            xxxxdold = xxxxd; 
                            TTTold = TTT;
                            RRRold = RRR;
                            validold = valid;
                           
                        end
                   
                        paranew          = paraold + cc*(sigscale*randn(npara,1));
%                        paranew          = paraold + cc*(sigscale*.001*randn(npara,1));
                       	[TTT,RRR,valid] = dsgesolv(paranew,mspec);
                        [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,paranew,nvar,nlags,mspec,npara);
						[postnew, likenew] = feval(strcat('objfcnmh9',inf),paranew,bounds,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode);
					    propdens         = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc^2) ...
                                        -0.5*(paranew - paraold)'*sigpropinv*(paranew - paraold)/cc^2;
						
						r = min([1 ; exp( postnew - postold)]);      

						if rand(1,1) < r;
						% Accept proposed jump
                        % disp('Accept proposed jump')
                        
                            parasim = paranew;
                            
							postsim(j,1) = postnew;
							likesim(j,1) = likenew;
							parasimm(j,:) = paranew';
							paraold = paranew;
							postold = postnew;
							likeold = likenew;
                            
                            yyyydold = yyyyd; 
                            xxyydold = xxyyd; 
                            xxxxdold = xxxxd;                                 

                            TTTold = TTT;
                            RRRold = RRR;
                            validold = valid;
						else;
						% Reject proposed jump
                        % disp('Reject proposed jump')
                            parasim = paranew;
                            
							likesim(j,1) = likeold;
							postsim(j,1) = postold;
							parasimm(j,:) = paraold';
							rej(j) = 1;
                            
                            yyyyd = yyyydold; 
                            xxyyd = xxyydold; 
                            xxxxd = xxxxdold; 
                            
                            TTT = TTTold;
                            RRR = RRRold;
                            valid = validold;
                        end
						if Verbose
                            fprintf(1,'\nBlock %2.2g of %2.2g',[iblock,nblocks]);
                            fprintf(1,'\nSimulation step %2.0f of %2.0f',[j,nsim]);
                            fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f ',[r,rej(j),sum(rej)/j]);
                            fprintf(1,'\nLikelihood: %4.6f Posterior: %4.6f Proposal Dens: %4.6f Estimated Score %4.6f ',[likesim(j,1),postsim(j,1),propsim(j,1),lnpy0+log(1/(sum(1/exp(likesim(1:j,1)-lnpy0))/j))]);
                            fprintf(1,'\n Parameters: %2.6f',parasim);
                        end                            
                    end
                    
                    
					%%%%%%%%%%%%%%%%%%%%%%%%% For each para, draw a CC and Sig for the VAR
                   
					if (lambdas > 0) & ~isinf(lambdas)
						
						yyyyd = YYYY + lambda*yyyyd;
						xxyyd = XXYY + lambda*xxyyd;
						xxxxd = XXXX + lambda*xxxxd;
                    elseif isinf(lambdas)
						
						yyyyd = yyyyd;
						xxyyd = xxyyd;
						xxxxd = xxxxd;
                    elseif lambdas == 0
						yyyyd = YYYY;
						xxyyd = XXYY;
						xxxxd = XXXX;

                    end
                    nobsc = nobs + lambda;    
					k     = 1+nlags*nvar;
					
					% Compute OLS estimates
					
                    invxxxxd = inv(xxxxd);
					c_hat    = invxxxxd*xxyyd;
					c_hat    = c_hat(:);
                                       
                    if ~isinf(lambdas)
						sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd)/nobsc;
						val     = 0;
						while ~val;
								
							% draw from the marginal posterior of sig :: Wishart Draws
							
							h = nobsc*sig_hat;           
							z = chol(inv(h))'*randn(nvar,nobsc-k);
							sig_sim = inv(z*z');
                            clear z h;
							
							% draw from the conditional posterior of C :: Normal Draws
							
							vc     = kron(sig_sim,invxxxxd);
							c_sim  = c_hat + chol(vc)'*randn(nvar*k,1);
							cct_sim= reshape(c_sim,k,nvar)';
							%clear xxxxd ;
                            
							% Check for stationarity			
							cc_tildes =  [cct_sim(:,(1+1):k) ; ...
                                       [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];
							if max(abs(eig(cc_tildes))) < 1; 
                    			val = 1;
                            end;
                            %clear cct_sim;
						
                        end; 
                        clear sig_hat;
                    elseif isinf(lambdas)
						sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd);
                        c_sim = c_hat;
                        sig_sim = sig_hat;
						cct_sim= reshape(c_sim,k,nvar)';
                    end
                    
                    clear sig_hat;
                    varpara(j,:) = [c_sim',sig_sim(:)'];
                    
                    %%%%%%%%%%%%%%%%%%%%%%%% Get rotation
	            if lambda > 0
	                aairf = irfdsge(TTT,RRR,parasimm(j,:)',nvar,mspec,npara,1,cum_irf);                    
        		    a0_m = [];
					for ii=1:nvar
						a0_m = [a0_m,aairf(1, nvar*(ii-1)+1:nvar*ii)'];   % On-Impact Matrix Given DSGE
					end; 
                    	[phimat,r_a] = qr(a0_m');         
                    	phimat = phimat';
                    	rotpara(j,:) = [phimat(:)'];
		        elseif lambda == 0
			        phimat = eye(nvar);
		        end;
                    
                    
					%%%%%%%%%%%%%%%%%%%%%%%%% For each cc and Sigma and rotation, get distribution of predicted values
                    sig_chol = chol(sig_sim)'*phimat;
                   
                    yypred  = zeros(qahead,nvar);
                    Shocks = randn(qahead,nvar);
		
                    xxpred =  XXpred;   % 1 | y(t), p(t), f(t) | y(t-1), p(t-1), f(t-1) | ...
                    for t = 1:qahead;		
                        out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')'; %  y(t+1), p(t+1), f(t+1)
						yypred(t,:) = out;
                        xxl = xxpred(1+1:k-nvar);
						xxpred = [1,out,xxl];   % 1 | y(t+1), p(t+1), f(t+1) | y(t), p(t), f(t) |...
                    
                    end;
                    

                    pred(j,:) = [yypred(:)'];
                    j/nsim+iblock-1                       

                end;    % END FOR LOOP OVER NSIMS
                               
                fwrite(fid1,parasimm, 'double');    % 
                fwrite(fid2,varpara,  'double');
                fwrite(fid3,pred,     'double');
                fwrite(fid4,postsim,  'double');
                fwrite(fid5,rotpara,  'double');
                
			end; 		% END FOR LOOP OVER NBLOCKS - given lambda and time period
toc
            num1 = ftell(fid1);
            fclose(fid1);
            num2 = ftell(fid2);
            fclose(fid2);
            num3 = ftell(fid3);
            fclose(fid3);
            num4 = ftell(fid4);
            fclose(fid4);
            num5 = ftell(fid5);
            fclose(fid5);
            
            outfile = strcat(spath,'/mhvarnums',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            save(outfile,'num1');	
            save(outfile,'num2','-append');	            
            save(outfile,'num3','-append');	            
            save(outfile,'num4','-append');	            
            
            
        %%% simply at mode    
        elseif JustMode
		                     

            %% open write files
            %% params file
            outfile = strcat(spath,'/mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid1 = fopen(outfile,'w');		% output file (set to write)   
            %% VAR file
            outfile = strcat(spath,'/varpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid2 = fopen(outfile,'w');		% output file (set to write)   
            %% Forecasts file
            outfile = strcat(spath,'/pred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
            fid3 = fopen(outfile,'w');		% output file (set to write)   

            
			%%%%%%%%%%%%%%%%%%%%%%%%% For mode params, draw a CC and Sig for the VAR
			if (lambdas > 0)  
               	[TTT,RRR,valid] = dsgesolv(params,mspec);
				[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara);
				
				yyyyd = YYYY + lambda*yyyyd;
				xxyyd = XXYY + lambda*xxyyd;
				xxxxd = XXXX + lambda*xxxxd;
            elseif isinf(lambdas)
				[yyyyd,xxyyd,xxxxd,retcode] = varprio(params,nvar,nlags,mspec,npara);

                yyyyd = yyyyd;
				xxyyd = xxyyd;
				xxxxd = xxxxd;
                
            else
				yyyyd = YYYY;
				xxyyd = XXYY;
				xxxxd = XXXX;
            end
	
            nobsc = nobs + lambda;    
			k     = 1+nlags*nvar;
			
			% Compute OLS estimates
			
			c_hat  = inv(xxxxd)*xxyyd;
        	sig_hat = (yyyyd -xxyyd'*inv(xxxxd)*xxyyd)/nobsc;
            
					
			%%%%%%%%%%%%%%%%%%%%%%%%% get predicted values
            yypred = yypredmex(XXpred,c_hat,qahead,nvar,k);
            
            fwrite(fid1,params','double');
            size([c_hat(:)',sig_hat(:)'])
            fwrite(fid2,[c_hat(:)',sig_hat(:)'],'double');
            ftell(fid2)
            fwrite(fid3,yypred','double');

            fclose(fid1);
            fclose(fid2);
            fclose(fid3);
                        
        end 
        


    end         % end lambdaseq
    
end % end fixparam
    
