function [ax,bx,cx,fa,fb,fc] = mnbrak(ax,bx,pcom,xicom,func,varargin)
% brackets min for a function func, given two intial points ax and bx (st func(ax)>func(bx))
% returns the 3 points of the bracketing, as well as the corresponding value fo the function
% taken from fortran routine with the same name (see Numerical Recipes by Press et al...)

% parameters
GOLD = 1.618034;
GLIMIT = 100;
TINY = 1e-20; %20

fa = f1dim(ax,pcom,xicom,func,varargin{:});
fb = f1dim(bx,pcom,xicom,func,varargin{:});
if fb > fa % reverse the two points
    dum = ax;
    ax = bx;
    bx = dum;
    dum = fa;
    fa = fb;
    fb = dum;
end

% guess c using golden rule 
cx = bx+GOLD*(bx-ax);
fc = f1dim(cx,pcom,xicom,func,varargin{:});

while fb >= fc % keep trying until you bracket: fb < fc
    % compute parabolic extrapolation from a,b,c
    r = (bx-ax)*(fb-fc);
    q = (bx-cx)*(fb-fa);
    si = sign(q-r);
    if si == 0
        %% parabolic does not have a chance: go straight to gold
        u = NaN;    
    else
        u = bx - ((bx-cx)*q-(bx-ax)*r)/(2*si*max(abs(q-r),TINY)); 
    end
    ulim = bx+GLIMIT*(cx-bx);
    
    if (bx-u)*(u-cx) > 0 %u is between b and c
        fu= f1dim(u,pcom,xicom,func,varargin{:});
        if fu < fc % success: the bracket points are b,u,c; just rename
            ax = bx;
            fa = fb;
            bx = u;
            fb = fu;
            return
        elseif fu > fb % success: the bracket points are a,b,u; just rename
            cx = u;
            fc = fu;
            return
        end
        % else: parabolic did not work
        u = cx+GOLD*(cx-bx);
        fu= f1dim(u,pcom,xicom,func,varargin{:});
    elseif (cx-u)*(u-ulim) > 0 %u is between c and the ulimit
        fu= f1dim(u,pcom,xicom,func,varargin{:});
        if fu < fc
            bx = cx;
            cx = u;
            u = cx+GOLD*(cx-bx);
            fb = fc;
            fc = fu;
            fu= f1dim(u,pcom,xicom,func,varargin{:});
        end
    elseif (u-ulim)*(ulim-cx) > 0 %u is beyond ulimit
        u = ulim;
        fu= f1dim(u,pcom,xicom,func,varargin{:});

    else
        u = cx+GOLD*(cx-bx);
        fu= f1dim(u,pcom,xicom,func,varargin{:});
    end
    % eliminate old points and continue
    ax = bx;
    bx = cx;
    cx = u;
    fa = fb;
    fb = fc;
    fc = fu;
end


        

        
