close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%% around line 140 is where to change the values for  policy parameters (polipar is defined in varspec)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
nobs =  nobsall;
XXall = [XXall;[1,YYall(end,:),XXall(end,1+1:1+nvar*nlags-nvar)]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% set date
I = find(ti == dates);


Verbose = 0;

date_q = (1:1:qahead)';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = 0;
fixparam = fixparamseq;
for lambdas = lambdaseq     
    
    L = L+1;
   		
    if stime                    
		YY     = YYall(I-stime+1:I,:);
		XX     = XXall(I-stime+1:I,:);             
    else
		YY     = YYall(1:I,:);
        XX     = XXall(1:I,:);                   
    end

    if I+qahead <= nobsall
		YYfut  = YYall(I+1:I+qahead,:);
        XXpred = XXall(I+1,:);
    elseif I == nobsall
		YYfut  = NaN*zeros(qahead,nvar);
        XXpred = XXall(end,:);
    elseif (I < nobsall) & (I+qahead > nobsall)
        Ii = nobsall-I
		YYfut  = [YYall(I+1:I+Ii,:);NaN*zeros(qahead-Ii,nvar)];
        XXpred = XXall(end-Ii,:);        
    end
	
    nobs = size(YY,1);

    lambda = lambdas*nobs;
    
	YYYY = YY'*YY;
	XXYY = XX'*YY;
	XXXX = XX'*XX;
    

    if isinf(lambdas)
        inf = 'inf';
    else
        inf = '';
    end
        
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
	fprintf(1,'\n Time %3.2f ',[ti(I)]);

    %% open write files
    %% params file
    infile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid1 = fopen(infile1,'r');		% output file (set to read)   
    %% VAR params
    infile20 = strcat(spath,'/varpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid20 = fopen(infile20,'r');		% output file (set to read)   
    %% Rotation file
    infile50 = strcat(spath,'/rot',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid50 = fopen(infile50,'r');		% output file (set to read)   


    %% VAR file
    outfile2 = strcat(spath,'/Avarpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid2 = fopen(outfile2,'w');		% output file (set to write)   
    %% Forecasts file
    outfile3 = strcat(spath,'/Apred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
    fid3 = fopen(outfile3,'w');		% output file (set to write)   
   
    %% set initial values for simulations
    
tic        
	for iblock = 1:nblocks  
        % read parameters from pre-Volcker and input non-policy parameters into para
        parasim = fread(fid1,[nsim,npara],'double');
        if any(size(parasim) ~= [nsim,npara])
   			fclose(fid1);
            break;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%% this is where to change the values for  policy parameters (polipar is defined in varspec)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ~isempty(polipar)
        	parasim(:,polipar) = repmat(polivalue',[nsim 1]);                             
        	%% parasim(:,polipar) = 1.33*parasim(:,polipar);                             

        	varpara = zeros(nsim,nvar*(1+nlags*nvar)+nvar^2);
		pred = zeros(nsim,nvar*qahead);
        elseif isempty(polipar)
                varpara = fread(fid20,[nsim,nvar*(1+nlags*nvar)+nvar^2],'double');        
	        rotpara = fread(fid50,[nsim,nvar^2],'double');

	end
            
        for j = 1:nsim                                        
	        if ~isempty(polipar)
         
        	   	[TTT,RRR,valid] = dsgesolv(parasim(j,:)',mspec);
			[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,parasim(j,:)',nvar,nlags,mspec,npara);
           
			if (lambdas > 0) & ~isinf(lambdas)
				
				yyyyd = YYYY + lambda*yyyyd;
				xxyyd = XXYY + lambda*xxyyd;
				xxxxd = XXXX + lambda*xxxxd;
            		elseif isinf(lambdas)
				
				yyyyd = yyyyd;
				xxyyd = xxyyd;
				xxxxd = xxxxd;
            		elseif lambdas == 0
				yyyyd = YYYY;
				xxyyd = XXYY;
				xxxxd = XXXX;

            		end
            		nobsc = nobs + lambda;    
			k     = 1+nlags*nvar;
			
			% Compute OLS estimates
			
            		invxxxxd = inv(xxxxd);
			c_hat  = invxxxxd*xxyyd;
			c_hat   = c_hat(:);
                               
            		if ~isinf(lambdas)
				sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd)/nobsc;
				val = 0;
				while ~val;
						
					% draw from the marginal posterior of sig
					
					h = nobsc*sig_hat;           
					z = chol(inv(h))'*randn(nvar,nobsc-k);
					sig_sim = inv(z*z');
                    			clear z h;
					
					% draw from the conditional posterior of C
					
					vc     = kron(sig_sim,invxxxxd);
					c_sim  = c_hat + chol(vc)'*randn(nvar*k,1);
					cct_sim= reshape(c_sim,k,nvar)';
					%clear xxxxd ;
                    
					% Check for stationarity			
					cc_tildes =  [cct_sim(:,(1+1):k) ; ...
                               [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];
					if max(abs(eig(cc_tildes))) < 1; 
            					val = 1;
                    			end;
                    		%clear cct_sim;
				
                		end; 
                		clear sig_hat;
            		elseif isinf(lambdas)
				sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd);
                		c_sim = c_hat;
                		sig_sim = sig_hat;
				cct_sim= reshape(c_sim,k,nvar)';
            		end

            		clear sig_hat;
            		varpara(j,:) = [c_sim',sig_sim(:)'];
            
		        %% get rotation
%            		aairf = irfdsge(TTT,RRR,parasim(j,:)',nvar,mspec,npara,1,cum_irf);                    
%       		a0_m = [];
%			for ii=1:nvar
%				a0_m = [a0_m,aairf(1,nvar*(ii-1)+1:nvar*ii)'];
%			end; 
%            		[phimat,r_a] = qr(a0_m');         
%            		phimat = phimat';
            		phimat = eye(nvar);
	        elseif isempty(polipar)
            		cct_sim = reshape(varpara(j,1:nvar*(1+nlags*nvar)),1+nlags*nvar,nvar)'; 
                        sig_sim = reshape(varpara(j,nvar*(1+nlags*nvar)+1:end),nvar,nvar);  
	                phimat = reshape(rotpara(j,:),nvar,nvar);  

			k     = 1+nlags*nvar;
		end

        	%%%%%%%%%%%%%%%%%%%%%%%%% For each cc and Sigma and rotation, get VAR identified impulse responses
            sig_chol = chol(sig_sim)'*phimat;
           
            yypred  = zeros(qahead,nvar);
            Shocks = randn(qahead,nvar);

            xxpred =  XXpred;
            for t = 1:qahead;		
                out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')';
				yypred(t,:) = out;
                xxl = xxpred(1+1:k-nvar);
				xxpred = [1,out,xxl];
            
            end;

            pred(j,:) = yypred(:)';

        end;
                       
        fwrite(fid2,varpara,'double');
        fwrite(fid3,pred,'double');
        
	end;		% end simulation - given lambda and time period
toc
    num1 = ftell(fid1);
    fclose(fid1);
    fclose(fid20);
    fclose(fid50);
    num2 = ftell(fid2);
    fclose(fid2);
    num3 = ftell(fid3);
    fclose(fid3);

end         % end lambdaseq
    
