function [p,fret] = powell(p,xi,ftol,ITMAX,func,varargin)
% given starting point p (nx1), and initial direction matrix (npxnp) - usually I(n), 
% a function func that needs to be minimized, powell performs Powell method
% ftol is the convergence tolerance. returns p (minimum, if attained) and corresponding value of func (fret)
% taken from fortran routine with the same name (see Numerical Recipes by Press et al...)

% parameters
NMAX = 50;
if isempty(ITMAX)
    ITMAX = 200; % max number of iterations, change if needed
end
Verbose = 1;

n = length(p);
fret = eval([func '(p,varargin{:})']);

pt = p; % save initial point

for iter = 1:ITMAX
	% Initialize
	ibig = 0;
	del = 0;
	fp = fret;
    
    % in each iteration, loop over all directions
    for i = 1:n;
        xit = xi(:,i);
        fptt = fret;
        [p,fret] = linmin(p,xit,func,varargin{:});
        if abs(fptt-fret) > del % record the largest decrease so far
            del = abs(fptt-fret);
            ibig = i;
        end
    end
    if 2*abs(fret-fp) <= ftol*(abs(fret)+abs(fp)) % test if done        
        if Verbose; disp(sprintf('Powell done, iteration: %2.2f, improvement: %20.10f',[iter,abs(fret-fp)])); end;
        return;
    end
    % construct extrapolated point and save old starting point
    ptt = 2*p-pt;
    xit = p-pt;
    pt = p;
    fptt = eval([func '(ptt,varargin{:})']);
    if ( (fptt >= fp) | (2*(fp-2*fret+fptt)*((fp-fret-del).^2)-del*(fp-fptt).^2 >= 0)) % don't use new direction
    else
        [p,fret] = linmin(p,xit,func,varargin{:});
        xi(:,ibig) = xi(:,n);
        xi(:,n) = xit;
    end  
    if Verbose; disp(sprintf('iteration: %2.2f, improvement: %20.10f',[iter,abs(fret-fp)])); end;
end

disp('powell exceeded maximum number of iterations');
%pause(5)