function [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara);

% description: The program converts the cross moment matrices for the DSGE prior


retcode = 1;

%% [TTT, RRR, valid] = dsgesolv(para,mspec);


if valid < 1;
	retcode = 0;
	
	yyyyd = zeros(nvar,nvar);
	xxyyd = zeros(1+nlags*nvar,nvar);
	xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
	
	return
end

nstate = size(TTT,1);

%% create system matrices for state space model
%% see dsgesolv for composition of state vector
ZZ = zeros(nvar,nstate);

if mspec == 2
	sigeps = para(8)^2;
	sigeta = para(9)^2;
	gam    = para(3);
	mst    = exp(para(4));	   
    
    %% state space: [dm(t-1), dp(t-1), dy(t-1), dm(t), dp(t), dk(t+1), dh(t), dy(t), dd(t+1), dq(t+1), dqt(t+1), dqb(t+1)
    %% output growth
    ZZ(1,8) = 1; ZZ(1,3) = -1;
    %% inflation
	ZZ(2,5) = 1; ZZ(2,2) = -1; ZZ(2,1) = 1;
	DD = [gam;(log(mst)-gam)];
	
	HH = [[ sigeps , -sigeps ];[ -sigeps , sigeps]];
	QQ = [[ sigeps , 0        ];[ 0      , sigeta]]; 
	VV = [[ sigeps , -sigeps ];[ 0       , 0      ]];

elseif ( mspec >= 3 ) & ( mspec <= 6 )
	sigeps = para(8)^2;
	sigeta = para(9)^2;
	gam    = para(3);
	lnpist    = para(4);

    %% state space: [dm(t-1), dp(t-1), dy(t-1), dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    %% output growth
    ZZ(1,8) = 1; ZZ(1,3) = -1;
    %% inflation
	ZZ(2,5) = 1; ZZ(2,2) = -1; ZZ(2,1) = 1;
	DD = [gam;lnpist];
	
	HH = [[ sigeps , -sigeps ];[ -sigeps , sigeps]];
	QQ = [[ sigeps , 0        ];[ 0      , sigeta]]; 
	VV = [[ sigeps , -sigeps ];[ 0       , 0      ]];    
    
elseif ( mspec == 101 )
	sigr = para(11);
	sigg = para(12);
	sigz = para(13);
	lngamm = para(1);
	lnpist = para(2);
	lnr2st = para(3)+lnpist;   %%% CHANGE
    
    %% state space: [y,inf,r2,r2star,g,z,xi_y,xi_infl,y-1]
		
    %% output growth
		       ZZ(1,1) = 1; ZZ(1,9) = -1; ZZ(1,6) = 1; %%% CHANGE
    %% inflation
	ZZ(2,2) = 1; 
    %% interest rate
	ZZ(3,3) = 4;
	DD = [lngamm;lnpist;4*lnr2st];
	
	HH = zeros(nvar,nvar);
%%	HH(1,1) = sigz^2;   %% CHANGE
	
	QQ = diag([sigr^2;sigg^2;sigz^2]);
	
	VV = zeros(nvar,nvar);
%%	VV(3,1) = sigz^2;    %% CHANGE
    
end

% Compute nlags autocovariances


% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlags+1);

% GA0 = RRR*QQ*RRR';
% GA0 =  GA0(:);
% GA0  = inv(eye(nstate^2) - kron(TTT,TTT))*GA0;
% GA0  = reshape(GA0',nstate,nstate)';
% GA0_ = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
GA0 = dlyap(TTT,RRR*QQ*RRR');


Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
Gl = Gl';
GAMM0(:,1) = Gl(:);

TTl = TTT;
for l = 1:nlags;
	Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
    Gl = Gl';
	GAMM0(:,l+1) = Gl(:);            %%%????
	TTl = TTl*TTT;
end

% Create limit cross product matrices

yyyyd = reshape(GAMM0(:,1)',nvar,nvar)' + DD*DD';

xxyyd = DD;
for l = 1:nlags;
    xxyyd = [xxyyd , (reshape(GAMM0(:,l+1)',nvar,nvar)' + DD*DD')];
end
xxyyd = xxyyd';

xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
xxxxd(1,1) = 1;
xxxxd(1,2:1+nlags*nvar) = kron(ones(1,nlags),DD');
xxxxd(2:1+nlags*nvar,1) = kron(ones(nlags,1),DD);

for r = 1:nlags;
	for l = r:nlags;	
		xxxxd(1+nvar*(r-1)+1:1+nvar*r,1+nvar*(l-1)+1:1+nvar*l) = reshape(GAMM0(:,l-r+1)',nvar,nvar)'+DD*DD';
		xxxxd(1+nvar*(l-1)+1:1+nvar*l,1+nvar*(r-1)+1:1+nvar*r) = reshape(GAMM0(:,l-r+1)',nvar,nvar)+DD*DD';
	end
end
  
    
