
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
nobs =  nobsall;

fixparam = fixparamseq(1);
if fixparam ~= 0 
    fixparam
    break
end

%% set date
Idate = find(ti == dates);
tiall     = [ti(1:end-1);(ti(end):0.25:(ti(end)+.25*qahead))'];
datesall = strcat(num2str(floor(tiall)),'-',num2str(round(1+4*(tiall-floor(tiall)))));  




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      forecasts 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% settings
percent = .68;
L = 0;

%% compute forecasts means and bands
Means = zeros(nvar,qahead,length(lambdaseq));
Bands = zeros(2,nvar,qahead,length(lambdaseq));

for lambdas = lambdaseq     
    L = L+1;
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    
    %% open read path files 
    infile = strcat(spath,'/spred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode));
    num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
    numb = nburn*nvar*qahead*8;                  % number of bites to dicard


    fid = fopen(infile,'r');		% input file (set to read)               
    status = fseek(fid,numb,'bof');

    ypath = [];
	while ( ftell(fid) < num ) 
        % Read blocks of size nsim
		ypathadd = fread(fid,[nsim,nvar*qahead],'double');
        if all(size(ypathadd,2) == [nvar*qahead])
            ypath = [ypath;ypathadd];
        else
   			fclose(fid);
        end
        clear ypathdd;
	
	end;
	fclose(fid);
    ndraws = size(ypath,1);

    for I = 1:nvar
        yp = ypath(:,qahead*(I-1)+1:qahead*I);
        yp = [1+3*cum_for(I)]*yp;
        
		Means(I,:,L) = mean(yp,1);
		Bands(:,I,:,L) = hpdint(yp,percent,1);

    end

end         % end lambdaseq

Alt = menu('Do you have an alternative forecast scenario?','Yes','No')
if Alt == 1
	%% compute forecasts means and bands for alternative forecast scenario 
	AMeans = zeros(nvar,qahead,length(lambdaseq));
	ABands = zeros(2,nvar,qahead,length(lambdaseq));
    
    L = 0;
	for lambdas = lambdaseq     
        L = L+1;
        
        fprintf(1,'\n Lambda = %2.3f',[lambdas]);
        
        %% open read path files 
        infile = strcat(spath,'Aspred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(Idate)),num2str(fixparam),num2str(JustMode));
        num = nblocks*nsim*nvar*qahead*8;            % number of bites per file
        numb = nburn*nvar*qahead*8;                  % number of bites to dicard
	
	
        fid = fopen(infile,'r');		% input file (set to read)               
        status = fseek(fid,numb,'bof');
	
        ypath = [];
		while ( ftell(fid) < num ) 
            % Read blocks of size nsim
			ypathadd = fread(fid,[nsim,nvar*qahead],'double');
            if all(size(ypathadd,2) == [nvar*qahead])
                ypath = [ypath;ypathadd];
            else
       			fclose(fid);
            end
            clear ypathdd;
		
		end;
		fclose(fid);
        ndraws = size(ypath,1);
	
        for I = 1:nvar
            yp = ypath(:,qahead*(I-1)+1:qahead*I);
            yp = [1+3*cum_for(I)]*yp;
			AMeans(I,:,L) = mean(yp,1);
			ABands(:,I,:,L) = hpdint(yp,percent,1);
	
        end
	
	end         % end lambdaseq
    
end

%% do plots

previous = 4+round(1+4*(ti(Idate)-floor(ti(Idate))));
future = 8+4-round(1+4*(ti(Idate)-floor(ti(Idate))));
sirf = (Idate-previous+1:1:Idate+future);
intx = datesall(sirf,:);
intx = intx(1:4:end,:);

titlenames = varnames;


%% set YLim
ylims = zeros(nvar,2);
ylims(1,:) = [-2 7];        %% output
ylims(2,:) = [-1 5];        %% infl
ylims(3,:) = [0 7];        %% fed fund

ylims = []

Ys = YYall(1:Idate,:);
Ys = repmat([1+3*cum_for],[Idate 1]).*Ys;

L = 0;
for lambdas = lambdaseq     
    L = L+1;
    
    
    for V = 1:nvar
        figure(100);
        subplot(length(lambdaseq),nvar,V+nvar*(L-1));   
        mlp = [Ys(:,V);squeeze(Means(V,:,L))'];
        bandsp = [repmat(Ys(:,V),[1 2]);squeeze(Bands(:,V,:,L))'];
        
        P = plot(sirf,mlp(sirf),'r-');
        set(P,'LineWidth',2.2);
        hold on;
        P = plot(sirf,bandsp(sirf,1),'r:',sirf,bandsp(sirf,2),'r:');
        set(P,'LineWidth',1);
        if Alt == 1
            
            Amlp = [Ys(:,V);squeeze(AMeans(V,:,L))'];
            Abandsp = [repmat(Ys(:,V),[1 2]);squeeze(ABands(:,V,:,L))'];
            
            P = plot(sirf,Amlp(sirf),'b-.');
            set(P,'LineWidth',1.5);
            hold on;
            P = plot(sirf,Abandsp(sirf,1),'b-.',sirf,Abandsp(sirf,2),'b-.');
            set(P,'LineWidth',.75);
            
        end

        %% make actual black
        mlp = [Ys(:,V);NaN*squeeze(Means(V,:,L))'];
        P = plot(sirf,mlp(sirf),'k-');
        set(P,'LineWidth',2.2);
        
        hold off;
        
        if L == 1
            title(titlenames(V));
        end
        if V == 1 
            ylabel(strcat('\lambda = ',num2str(lambdas)));
        end
        if ~isempty(ylims)
            set(gca,'YLim',ylims(V,:));
        end
        
        set(gca,'XLim',[min(sirf),max(sirf)]);
        set(gca,'XTick',sirf(1:4:end));
        set(gca,'XTickLabel',intx);
        
    end   

end         % end lambdaseq

figure(100)
set(figure(100),'Color',[1 1 1]);
titletext = input('\n title for forecast plots? ');
[ax,h]=subtitle(titletext);

titlefig = input('\n figure title ');
ng = strcat(gpath,lmodel,lprior,dataset,new,st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(Idate)),num2str(JustMode),titlefig,'for');
saveas(figure(100),ng,'fig');

set(figure(100),'PaperPosition',[0 0 8 6])
print(char(strcat(ng,'.eps')),'-deps');

%saveas(figure(100),strcat(gpath,num2str(100*ti(Idate)),'for'),'fig');



pause;
close all;
