close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%% check
if ( ( mspec <= 6 ) & (dataset-floor(dataset)) ) | ( ( mspec == 101 ) & ~(dataset-floor(dataset)) ) 
    fprintf(1,'problem: check consistency of  model (%2.0f) and dataset (%2.1f)',[mspec,dataset]); break;
end

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = strcat(spath,'dataout',dataset);
load(fname,'nobsall')
load(fname,'ti')
load(fname,'YYall')
load(fname,'XXall')
XXall = [XXall;[1,YYall(end,:),XXall(end,1+1:1+nvar*nlags-nvar)]];
nobs =  nobsall;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nirf = 41;
%% cumulative irf
ccum_irf = [];
for II=1:length(cum_irf)
    ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end

%% set date
I = find(ti == dates);

%%  Configure the Metropolis Algorithm
if JustMode
    nblocks = 1;
    nsim = 1;
end


Verbose = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for Alt = [1]; %menu('','baseline','alternative');
Alt = Alt-1
fixparam = 0;
    for lambdas = lambdaseq     
        
        fprintf(1,'\n Lambda = %2.3f',[lambdas]);
		fprintf(1,'\n Time %3.2f ',[ti(I)]);
            
        if stime                    
			YY     = YYall(I-stime+1:I,:);
			XX     = XXall(I-stime+1:I,:);             
        else
			YY     = YYall(1:I,:);
            XX     = XXall(1:I,:);                   
        end
	
        if I+qahead <= nobsall
			YYfut  = YYall(I+1:I+qahead,:);
            XXpred = XXall(I+1,:);
        elseif I == nobsall
			YYfut  = NaN*zeros(qahead,nvar);
            XXpred = XXall(end,:);
        elseif (I < nobsall) & (I+qahead > nobsall)
            Ii = nobsall-I
			YYfut  = [YYall(I+1:I+Ii,:);NaN*zeros(qahead-Ii,nvar)];
            XXpred = XXall(end-Ii,:);        
        end
		
        nobs = size(YY,1);
	
        lambda = lambdas*nobs;
        
		YYYY = YY'*YY;
		XXYY = XX'*YY;
		XXXX = XX'*XX;
        
	
        if isinf(lambdas)
            inf = 'inf';
        else
            inf = '';
        end
            
        nobsc = nobs + lambda;    
		k     = 1+nlags*nvar;

        %% open write files
        %% params file
        infile1 = strcat(spath,'mhpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
        fid1 = fopen(infile1,'r');		% output file (set to read)   
        num1 = nblocks*nsim*npara*8;            % number of bites per file
        
        %% VAR file
        infile2 = strcat(spath,'varpara',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
        fid2 = fopen(infile2,'r');		% output file (set to read)   
        num2 = nblocks*nsim*(nvar*(1+nlags*nvar)+nvar^2)*8;            % number of bites per file
        
        %% Rotation file
        infile5 = strcat(spath,'/rot',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));		
        fid5 = fopen(infile5,'r');		% output file (set to write)   
        
        
        %% Alt forecast path
        outfile4 = strcat(spath,'/spred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));
        if Alt
            outfile4 = strcat(spath,'/Aspred',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));            
        end
        fid4 = fopen(outfile4,'w');		% output file (set to write)   


        %% identified residuals
        outfile55 = strcat(spath,'/resid',lmodel,lprior,dataset,new,st,num2str(100*lambdas),num2str(100*ti(I)),num2str(fixparam),num2str(JustMode));
        fid55 = fopen(outfile55,'w');		% output file (set to write)   
        
        %% set initial values for simulations
        
	for iblock = 1:nblocks  
        

            varpara = fread(fid2,[nsim,nvar*(1+nlags*nvar)+nvar^2],'double');        
            parasimm = fread(fid1,[nsim,npara],'double');
            rotpara = fread(fid5,[nsim,nvar^2],'double');

    		pred = zeros(nsim,nvar*qahead);
    		ires = zeros(nsim,nvar*nobs);
                
            for j = 1:size(varpara,1)
                params = parasimm(j,:)';
                
                cct_sim = reshape(varpara(j,1:nvar*(1+nlags*nvar)),1+nlags*nvar,nvar)'; 
                sig_sim = reshape(varpara(j,nvar*(1+nlags*nvar)+1:end),nvar,nvar);  
                phimat = reshape(rotpara(j,:),nvar,nvar);  
                  
    			%%%%%%%%%%%%%%%%%%%%%%%%% For each cc and Sigma and rotation, get VAR identified impulse responses
                sig_chol = chol(sig_sim)'*phimat;

		%% compute historical identified shocks
		isig_chol = inv(sig_chol)';
		uresid = YY-XX*cct_sim';
		iresid = uresid*isig_chol;

		ires(j,:) = iresid(:)';


                yypred  = zeros(qahead,nvar);
                Shocks = randn(qahead,nvar);
                Shocks(:,1) = 0;

                xxpred =  XXpred;

                if Alt
                    for t = 1:qahead;
                        out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')';
                        target = 1;
                        if ( out(3) > target ) & ( t < 3 )
                            mshock = (target - cct_sim(3,:)*xxpred'-sig_chol(3,2:3)*Shocks(t,2:3)')/sig_chol(3,1);
                            out = (cct_sim*xxpred'+sig_chol*[mshock Shocks(t,2:3)]')';
                        end
                        
						yypred(t,:) = out;
                        xxl = xxpred(1+1:k-nvar);
						xxpred = [1,out,xxl];
                    
                    end;
                else
                    for t = 1:qahead;		
                        out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')';
						yypred(t,:) = out;
                        xxl = xxpred(1+1:k-nvar);
						xxpred = [1,out,xxl];
                    
                    end;
                end  
                
                pred(j,:) = yypred(:)';
                
            end;
          
            fwrite(fid4,pred,'double');
            fwrite(fid55,ires,'double');
		end;		% end simulation - given lambda and time period
        fclose(fid1);
        fclose(fid2);
        fclose(fid5);
        fclose(fid4);
        fclose(fid55);
            
    end         % end lambdaseq
end % end Alt
    
    

