function [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara);

% description: The program converts the cross moment matrices for the DSGE prior


retcode = 1;

%% [TTT, RRR, valid] = dsgesolv(para,mspec);


if valid < 1;
	retcode = 0;
	
	yyyyd = zeros(nvar,nvar);
	xxyyd = zeros(1+nlags*nvar,nvar);
	xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
	
	return
end

nstate = size(TTT,1);

%% create system matrices for state space model
%% see dsgesolv for composition of state vector
ZZ = zeros(nvar,nstate);

if mspec == 2
	sigeps = para(8)^2;
	sigeta = para(9)^2;
	gam    = para(3);
	mst    = exp(para(4));	   
    
    %% state space: [dm(t-1), dp(t-1), dy(t-1), dm(t), dp(t), dk(t+1), dh(t), dy(t), dd(t+1), dq(t+1), dqt(t+1), dqb(t+1)
    %% output growth
    ZZ(1,8) = 1; ZZ(1,3) = -1;
    %% inflation
	ZZ(2,5) = 1; ZZ(2,2) = -1; ZZ(2,1) = 1;
	DD = [gam;(log(mst)-gam)];
	
	HH = [[ sigeps , -sigeps ];[ -sigeps , sigeps]];
	QQ = [[ sigeps , 0        ];[ 0      , sigeta]]; 
	VV = [[ sigeps , -sigeps ];[ 0       , 0      ]];

elseif ( mspec >= 3 ) & ( mspec <= 6 )
	sigeps = para(8)^2;
	sigeta = para(9)^2;
	gam    = para(3);
	lnpist    = para(4);

    %% state space: [dm(t-1), dp(t-1), dy(t-1), dm(t), dp(t), dk(t+1), dh(t), dy(t), dq(t+1), dqt(t+1), dqb(t+1), R(t)]
    %% output growth
    ZZ(1,8) = 1; ZZ(1,3) = -1;
    %% inflation
	ZZ(2,5) = 1; ZZ(2,2) = -1; ZZ(2,1) = 1;
	DD = [gam;lnpist];
	
	HH = [[ sigeps , -sigeps ];[ -sigeps , sigeps]];
	QQ = [[ sigeps , 0        ];[ 0      , sigeta]]; 
	VV = [[ sigeps , -sigeps ];[ 0       , 0      ]];    
    
elseif ( mspec == 101 )
	sigr = para(11);
	sigg = para(12);
	sigz = para(13);
	lngamm = para(1);
	lnpist = para(2);
	lnr2st = para(3)+lnpist;   %%% CHANGE
    
    %% state space: [y,inf,r2,r2star,g,z,xi_y,xi_infl,y-1]
		
    %% output growth
		       ZZ(1,1) = 1; ZZ(1,9) = -1; ZZ(1,6) = 1; %%% CHANGE
    %% inflation
	ZZ(2,2) = 1; 
    %% interest rate
	ZZ(3,3) = 4;
	DD = [lngamm;lnpist;4*lnr2st];
	
	HH = zeros(nvar,nvar);
%%	HH(1,1) = sigz^2;   %% CHANGE
	
	QQ = diag([sigr^2;sigg^2;sigz^2]);
	
	VV = zeros(nvar,nvar);
%%	VV(3,1) = sigz^2;    %% CHANGE
    
end

% Compute nlags autocovariances


% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlags+1);

% GA0 = RRR*QQ*RRR';
% GA0 =  GA0(:);
% GA0  = inv(eye(nstate^2) - kron(TTT,TTT))*GA0;
% GA0  = reshape(GA0',nstate,nstate)';
% GA0_ = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
% GA0 = dlyap(TTT,RRR*QQ*RRR');
a=TTT; c=RRR*QQ*RRR';
[m,n] = size(a);
a = (a+eye(m))\(a-eye(m));
c = (eye(m)-a)*c*(eye(m)-a')/2;
GA0 = lyap(a,c);

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
Gl = Gl';
GAMM0(:,1) = Gl(:);

TTl = TTT;
for l = 1:nlags;
	Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
    Gl = Gl';
	GAMM0(:,l+1) = Gl(:);            %%%????
	TTl = TTl*TTT;
end

% Create limit cross product matrices

yyyyd = reshape(GAMM0(:,1)',nvar,nvar)' + DD*DD';

xxyyd = DD;
for l = 1:nlags;
    xxyyd = [xxyyd , (reshape(GAMM0(:,l+1)',nvar,nvar)' + DD*DD')];
end
xxyyd = xxyyd';

xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
xxxxd(1,1) = 1;
xxxxd(1,2:1+nlags*nvar) = kron(ones(1,nlags),DD');
xxxxd(2:1+nlags*nvar,1) = kron(ones(nlags,1),DD);

for r = 1:nlags;
	for l = r:nlags;	
		xxxxd(1+nvar*(r-1)+1:1+nvar*r,1+nvar*(l-1)+1:1+nvar*l) = reshape(GAMM0(:,l-r+1)',nvar,nvar)'+DD*DD';
		xxxxd(1+nvar*(l-1)+1:1+nvar*l,1+nvar*(r-1)+1:1+nvar*r) = reshape(GAMM0(:,l-r+1)',nvar,nvar)+DD*DD';
	end
end
  
    
function X = lyap(A, B, C)
%LYAP  Solve continuous-time Lyapunov equations.
%
%   X = LYAP(A,C) solves the special form of the Lyapunov matrix 
%   equation:
%
%           A*X + X*A' = -C
%
%   X = LYAP(A,B,C) solves the general form of the Lyapunov matrix
%   equation (also called Sylvester equation):
%
%           A*X + X*B = -C
%
%   See also  DLYAP.

%	S.N. Bangert 1-10-86
%	Copyright 1986-2002 The MathWorks, Inc. 
%	$Revision: 1.11 $  $Date: 2002/04/04 15:18:47 $
%	Last revised JNL 3-24-88, AFP 9-3-95

ni = nargin;

if ni==2,
   C = B;
   B = A';
end

[ma,na] = size(A);
[mb,nb] = size(B);
[mc,nc] = size(C);

% A and B must be square and C must have the rows of A and columns of B
if (ma ~= na) | (mb ~= nb) | (mc ~= ma) | (nc ~= mb)
   error('Dimensions do not agree.')
elseif ma==0 | mb==0,
   X = zeros(ma,mb);
   return
end

% Perform schur decomposition on A (and convert to complex form)
[ua,ta] = schur(A);
[ua,ta] = rsf2csf(ua,ta);
if ni==2,
   % Schur decomposition of A' can be calculated from that of A.
   j = ma:-1:1;
   ub = ua(:,j);
   tb = ta(j,j)';
else
   % Perform schur decomposition on B (and convert to complex form)
   [ub,tb] = schur(B);
   [ub,tb] = rsf2csf(ub,tb);
end

% Check all combinations of ta(i,i)+tb(j,j) for zero
p1 = diag(ta).'; % Use .' instead of ' in case A and B are not real
p1 = p1(ones(mb,1),:);
p2 = diag(tb);
p2 = p2(:,ones(ma,1));
sum = abs(p1) + abs(p2);
if any(any(sum == 0)) | any(any(abs(p1 + p2) < 1000*eps*sum))
   error('Solution does not exist or is not unique.');
end

% Transform C
ucu = -ua'*C*ub;

% Solve for first column of transformed solution
y = zeros(ma,mb);
ema = eye(ma);
y(:,1) = (ta+ema*tb(1,1))\ucu(:,1);

% Solve for remaining columns of transformed solution
for k=2:mb,
   km1 = 1:(k-1);
   y(:,k) = (ta+ema*tb(k,k))\(ucu(:,k)-y(:,km1)*tb(km1,k));
end

% Find untransformed solution 
X = ua*y*ub';

% Ignore complex part if real inputs (better be small)
if isreal(A) & isreal(B) & isreal(C),
   X = real(X);
end

% Force X to be symmetric if ni==2 and C is symmetric
if (ni==2) & isequal(C,C'),
   X = (X+X')/2;
end

