% This file contains the specification of the VAR model
% OUTPUT
% mspec:        code of the DSGE model (<100 2 exogenous variables, >=100 3 exogenous variables)
% mprior:       code for the choice of the prior
% dataset:      code for the data set (( used in pathspec (floor(404.2)) ))
% dates:        end of the sample
% lambdaseq:    vector of values of lambda
% fisparamseq:  0/1 stochastic/fixed parameters
% new:          controls the type of priors ('new' is the default value)
% JustMode:     0/1 do not / compute forecast with simulations (at the mode only)
% qahead:       quarters ahead for computing forecasts

% Configure the Metropolis Algorithm (number of simulations)
% nblocks:      
% nsim:         nbolcks*nsim=total simulations
% nburn:        number of burn-in chain draws

% stime:        n. obs. to use in VAR
% st:           string = stime
% nlags:        number of lags
% T0:           last observation discarded
% Tend:         0 to the end of the data set
% npara:        number of parameters (vd. mspec)
% para_names:   sting, names of the parameters (vd. mspec)
% bounds:       bounds of the parameter space
% para:         initial values of parameters (starting point for the MCMC posterior simulator)

% varnames:     names of shocks, and variables for which we want cumulative forecasts
% names_shocks: names of shocks, and variables for which we want cumulative irf
% cum_for:      [1 1]/[1 1 0] for mspec <100 / >=100
% cum_irf:      

% identify policy and non policy parameters
% polipar:      = 6, w/ mspec=101
% polivalue:    = 1, w/ mspec=101

function [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,new,JustMode,stime,st,nblocks,nsim,nburn,qahead] = varspec;


%% choose model
mspec = 101;
%% choose prior
mprior = 2; %% 2 is standard
%% choose dataset 
dataset = 404.2;
%dataset = 304.2
%% choose date
dates = 2001.5;
%dates = 2004.00;
%dates = 2003.75
%% Effective sample size to generate prior 
%lambdaseq = [0.5];   
lambdaseq=[0.2 0.6 1 1.4 1.8 10 Inf];
% lambdaseq=[ 0.6 ]



%% fixparamseq = 1 exogenous (fixed parameters) at para
fixparamseq = [0];
%% new controls the type of prior (normalization), 'new' is the default
new = 'new';
%% if JustMode: do not do simulations and compute forecasts at mode
JustMode = 0;           
% quarters ahead for computing forecasts
qahead = 16;                                     

%%  Configure the Metropolis Algorithm (number of simulations)
nblocks = 10; %1
%%nsim    = 2500 %2500; %100
nsim=100;
nburn = nsim;                                       % initial simulations to 'burn'

%% number of observations used in VAR (st = [] means use entire sample) 
stime = 80; 
st = num2str(stime);

%% number of lags
nlags = 4;           

%% End of Presample
T0 = nlags;
Tend     = 0;        % End of Estimation Sample 

%% number of endogenous variables 
if mspec < 100
    nvar = 2;        
else
    nvar = 3;        
end

% number of parameters in DSGE model 
if mspec < 3 
    npara = 10;
elseif ( mspec >= 3 ) & ( mspec <= 6 ) 
    npara = 12;
elseif ( mspec == 101 ) | ( mspec == 102 )
    npara = 13;          
end

% names of parameters 
if mspec < 3 
    para_names = { 'ALP';'BET';'GAM';'LNMST';'RHO';'PSI';'DEL';'SIGEPS';'SIGETA';'KAP'};
elseif ( mspec >= 3 ) & ( mspec <= 6 ) 
    para_names = { 'ALP';'BET';'GAM';'LNPIST';'RHO';'PSI';'DEL';'SIGEPS';'SIGETA';'KAP';'PHI1';'PHI2'};
elseif ( mspec == 101 ) | ( mspec == 102 )
  	para_names = { 'lngamm';'lnpist';'lnrrst';'kap';'tau';'psi1';'psi2';'rhor';'rhog';'rhoz';'sigr';'sigg';'sigz'};
end

% bounds for MH
if mspec < 100 
    bounds = ...
    [[0.1  , 0.99999 ]; ...
    [0.5  , 0.99999 ]; ...
    [-0.5 , 0.5     ]; ...
    [-0.5 , 0.5     ]; ...
    [1E-5 , 0.99999 ]; ...
    [1E-5 , 0.99999 ]; ...
    [1E-5 , 0.99999 ]; ...
    [1E-6 , 100     ]; ...
    [1E-6 , 100     ]; ...
    [1E-7 , 500     ]];
    if mspec >= 3
        bounds = ...
            [bounds; ...
                [1E-7 , 100     ]; ...
                [1E-7 , 100     ]];
    end
elseif ( mspec == 101 ) | ( mspec == 102 )    
    bounds = ...
    [[-1,10 ]; ...
    [-1,10 ]; ...
    [1E-6,10 ]; ...
    [-10,10  ]; ...
    [1E-5,100 ]; ...
    [1E-5,10  ]; ...
    [1E-5,10  ]; ...
    [1E-6,0.9999999 ]; ...
    [1E-6,0.9999999 ]; ...
    [1E-6,0.9999999 ]; ...
    [1E-7,500     ]; ...
    [1E-7,500     ]; ...
    [1E-7,500     ]];
    
end


% starting values for parameters
if mspec < 100 
	alp     = 0.35; %
	bet     = 0.99; %
	gam     = 0.005; %
	lnmst   = 0.0002; 
	rho     = 0.13; %
	psi     = 0.65; %
	del     = 0.01; %
	sig_eps = 0.02; %
	sig_eta = 0.005; %
	kap  = 50; %
    para = [alp ; bet ; gam   ; lnmst ; rho ; psi ; del ; sig_eps ; sig_eta ; kap];
    if mspec >= 3
    	lnpist = 0.0100;
    	phi1 = 1; 
    	phi2 = 1;	
        para = [para(1:3,1); lnpist ;para(5:10,1); phi1; phi2];
    end
elseif ( ( mspec == 101 ) | ( mspec == 102 ) )  
    lngamm  = 0.5;
	lnpist  = 1;
	lnrrst  = 0.5;
	kap     = 0.4;
	tau     = 1;
	psi1    = 2.5;
	psi2    = 0.3;
	rhor    = 0.4;
	rhog    = 0.8;
	rhoz    = 0.2;
	sigr    = 0.5;
	sigg    = .5;
	sigz    =  1;
	para = [lngamm ; lnpist ; lnrrst ; kap ; tau ; psi1 ; psi2 ; ...
           rhor ; rhog ; rhoz ; sigr ; sigg ; sigz];
end

%% names of variables, and variables for which we want cumulative forecasts
%% names of shocks, and variables for which we want cumulative irf
if mspec <= 6 
    varnames = {'real GDP growth';'inflation'};
    cum_for = [1 1];
    cum_irf = [1];
    names_shocks = {'Tech.';'Money'};
elseif ( mspec == 101 ) | ( mspec == 102 )
    varnames = {'Real GDP growth';'Inflation Rate';'Fed Funds Rate'};
    cum_for = [1 1 0];
    cum_irf = [1];
    names_shocks = {'Money';'Gvmt';'Tech'};    
end

%% identify policy and non policy parameters
if mspec < 3 
    polipar = [5];
elseif ( mspec >= 3 ) & ( mspec <= 6 ) 
    polipar = [5 11 12];
elseif ( mspec == 101 ) | ( mspec == 102 )
    polipar = [6];
    polivalue = [1];
end